//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				Chodza.cpp
// Ucel suboru:			definicia triedy CChodza - trieda uchovava udaje o nacitanej chodzi, 
//						polohy klbov a hodnoty UR, a hodnoty okrajov pri vykreslovani do grafu
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Martin Soltis
// Naposledy zmenil:	Soltis / 20.4.2005
// Poznamka:			UR = uhlova rychlost
//______________________________________________________________________________________________________


#include "stdafx.h"
#include "chodza.h"


CChodza::CChodza(CGraf* pGraf, CString strMeno, int nFPS)
	: m_pGraf(pGraf)
	, m_strMeno(strMeno)
	, m_nFPS(nFPS)
{
}


CChodza::~CChodza(void)
{
}


// nastavenie hodnoty zadaneho okraju
void CChodza::NastavHodnotyOkrajov(int nOkraj, double fHodnota)
{
	ASSERT(nOkraj > -1 && nOkraj < POCET_HODNOT_OKRAJOV);
	m_fHodnotyOkrajov[nOkraj] = fHodnota;
}


// nastavenie hodnot vsetkych okrajov
void CChodza::NastavHodnotyOkrajov(double fHorny, double fDolny,
								   double fLavy, double fPravy,
								   double fHornyUR, double fDolnyUR)
{
	m_fHodnotyOkrajov[HORNY_O] = fHorny;
	m_fHodnotyOkrajov[DOLNY_O] = fDolny;
	m_fHodnotyOkrajov[LAVY_O] = fLavy;
	m_fHodnotyOkrajov[PRAVY_O] = fPravy;
	m_fHodnotyOkrajov[HORNY_O_UR] = fHornyUR;
	m_fHodnotyOkrajov[DOLNY_O_UR] = fDolnyUR;
}


// vracia hodnotu zadaneho okraju
double CChodza::HodnotaOkraju(int nOkraj) 
{
	ASSERT(nOkraj > -1 && nOkraj < POCET_HODNOT_OKRAJOV);
	return m_fHodnotyOkrajov[nOkraj];
}


// nastavenie vsetkych okrajov podla poloh klbov a hodnot UR tak,
// aby sa cely priebeh nachadzal medzi hodnotami okrajov
void CChodza::NastavHodnotyOkrajov()
{
	NastavHodnotyHorizontalnychOkrajov();
	NastavHodnotyOkrajovUR();
	NastavHodnotyCasovychOkrajov();
}


// nastavenie horneho a dolneho okraju podla poloh klbov
void CChodza::NastavHodnotyHorizontalnychOkrajov()
{
	int i = 0;
	// nastavenie sa na prvy klb, ktory je viditelny
	while (i < POCET_KLBOV && !m_pGraf->ViditelnostKlbu(i))
		i++;
	// ak nie je ziaden viditelny, nastavime to podla 0-teho klbu
	if (i == POCET_KLBOV) {
		m_fHodnotyOkrajov[HORNY_O] = NajdiKlbMin(0);
		m_fHodnotyOkrajov[DOLNY_O] = NajdiKlbMax(0);
	}
	// inak nastavime podla prveho viditelneho
	else {
		double	fMin, fMax, fHodn;
		fMin = NajdiKlbMin(i);
		fMax = NajdiKlbMax(i);
		// a prejdeme vsetky ostatne zoznamy viditelnych klbov
		for (i++; i < POCET_KLBOV; i++) {
			if (m_pGraf->ViditelnostKlbu(i)) {
				fHodn = NajdiKlbMax(i);
				if (fMax < fHodn)
					fMax = fHodn;
				fHodn = NajdiKlbMin(i);
				if (fMin > fHodn)
					fMin = fHodn;
			}
		}
		m_fHodnotyOkrajov[HORNY_O] = fMin;
		m_fHodnotyOkrajov[DOLNY_O] = fMax;
	}
}


// nastavenie horneho a dolneho okraju pre UR podla hodnot UR
void CChodza::NastavHodnotyOkrajovUR()
{
	int i = 0;
	// nastavenie sa na prvu UR, ktora je viditelna
	while (i < POCET_UR && !m_pGraf->ViditelnostUR(i))
		i++;
	// ak nie je ziadna viditelna, nastavime to podla 0-tej UR
	if (i == POCET_UR) {
		m_fHodnotyOkrajov[HORNY_O_UR] = NajdiURMin(0);
		m_fHodnotyOkrajov[DOLNY_O_UR] = NajdiURMax(0);
	}
	// inak nastavime podla prvej viditelnej
	else {
		double	fMin, fMax, fHodn;
		fMin = NajdiURMin(i);
		fMax = NajdiURMax(i);
		// a prejdeme vsetky ostatne zoznamy viditelnych UR
		for (i++; i < POCET_UR; i++) {
			if (m_pGraf->ViditelnostUR(i)) {
				fHodn = NajdiURMax(i);
				if (fMax < fHodn)
					fMax = fHodn;
				fHodn = NajdiURMin(i);
				if (fMin > fHodn)
					fMin = fHodn;
			}
		}
		m_fHodnotyOkrajov[HORNY_O_UR] = fMin;
		m_fHodnotyOkrajov[DOLNY_O_UR] = fMax;
	}
}


// nastavenie laveho a praveho okraja podla dlzky sledovanych priebehov
void CChodza::NastavHodnotyCasovychOkrajov()
{
	m_fHodnotyOkrajov[LAVY_O] = 0;
	m_fHodnotyOkrajov[PRAVY_O] = double(VelkostKlbu(POCET_KLBOV - 1) - 1) / m_nFPS;
}


// vracia prepocet pozicie x-ovej suradnice v grafe na x-ovu hodnotu v grafe (casova os)
double CChodza::ZistiHodnotuX(int nPozX) const
{
	double fCitatel = m_fHodnotyOkrajov[PRAVY_O] - m_fHodnotyOkrajov[LAVY_O];
	double fMenovatel = double(m_pGraf->Okraj(PRAVY_O)) - m_pGraf->Okraj(LAVY_O);
	double fNasobitel = double(nPozX - m_pGraf->Okraj(LAVY_O));

	return fNasobitel*fCitatel/fMenovatel + m_fHodnotyOkrajov[LAVY_O];
}


// vracia prepocet pozicie y-ovej suradnice v grafe na y-ovu hodnotu v grafe (zvisla os)
double CChodza::ZistiHodnotuY(int nPozY) const
{
	double fCitatel = m_fHodnotyOkrajov[DOLNY_O] - m_fHodnotyOkrajov[HORNY_O];
	double fMenovatel = double(m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O));
	double fNasobitel = double(nPozY - m_pGraf->Okraj(HORNY_O));
	
	return fNasobitel*fCitatel/fMenovatel + m_fHodnotyOkrajov[HORNY_O];
}


// vracia prepocet pozicie y-ovej suradnice v grafe na hodnotu UR v grafe (zvisla os)
double CChodza::ZistiHodnotuUR(int nPozY) const
{
	double fCitatel = m_fHodnotyOkrajov[DOLNY_O_UR] - m_fHodnotyOkrajov[HORNY_O_UR];
	double fMenovatel = double(m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O));
	double fNasobitel = double(nPozY - m_pGraf->Okraj(HORNY_O));
	
	return fNasobitel*fCitatel/fMenovatel + m_fHodnotyOkrajov[HORNY_O_UR];
}


// vracia prepocet hodnoty x na poziciu x v grafe (casova os)
int CChodza::ZistiPozX(double fHodX) const
{
	double fCitatel = double(m_pGraf->Okraj(PRAVY_O) - m_pGraf->Okraj(LAVY_O));
	double fMenovatel = m_fHodnotyOkrajov[PRAVY_O] - m_fHodnotyOkrajov[LAVY_O];
	double fNasobitel = fHodX - m_fHodnotyOkrajov[LAVY_O];
	
	if (fCitatel == 0 || fMenovatel == 0)
		return m_pGraf->GetOs();
	
	return int(fNasobitel*fCitatel/fMenovatel + m_pGraf->Okraj(LAVY_O));
}


// vracia prepocet hodnoty y na poziciu y v grafe (zvisla os)
int CChodza::ZistiPozY(double fHodY) const
{
	double fCitatel = double(m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O));
	double fMenovatel = m_fHodnotyOkrajov[DOLNY_O] - m_fHodnotyOkrajov[HORNY_O];
	double fNasobitel = fHodY - m_fHodnotyOkrajov[HORNY_O];
	
	if (fCitatel == 0 || fMenovatel == 0)
		return m_pGraf->GetOs();
	
	return int(fNasobitel*fCitatel/fMenovatel + m_pGraf->Okraj(HORNY_O));
}


// vracia prepocet hodnoty UR na poziciu y v grafe (zvisla os)
int CChodza::ZistiPozUR(double fHodY) const
{
	double fCitatel = double(m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O));
	double fMenovatel = m_fHodnotyOkrajov[DOLNY_O_UR] - m_fHodnotyOkrajov[HORNY_O_UR];
	double fNasobitel = fHodY - m_fHodnotyOkrajov[HORNY_O_UR];
	
	if (fCitatel == 0 || fMenovatel == 0)
		return m_pGraf->GetOs();
	
	return int(fNasobitel*fCitatel/fMenovatel + m_pGraf->Okraj(HORNY_O));
}


// vracia vypis suradnic (hodnot x, y, UR) podla zadanej pozicie bodu
// parameter os sluzi na zadanie veliciny, ktora je sledovana na druhej
// (nie casovej) osi grafu
CString CChodza::GetSuradnice(CPoint pt, CString strOs) const
{
	CString strSuradnice;
	strSuradnice.Format("t: %.3lfs\n%s: %.1fpx\nur: %.0lf/s", 
		ZistiHodnotuX(pt.x), strOs, ZistiHodnotuY(pt.y), ZistiHodnotuUR(pt.y));

	return strSuradnice;
}


// nastavenie popisov pre vykreslovane priebehy
void CChodza::NastavMenovky(CDC* pDC)
{
	pDC->SetBkColor(m_pGraf->Farba(BIELA));
	NastavMenovkyKlbom(pDC);
	NastavMenovkyUR(pDC);
}


// nastavenie popisov pre vykreslovane priebehy klbov
void CChodza::NastavMenovkyKlbom(CDC* pDC)
{
	for (int i = 0; i < POCET_KLBOV; i++)
		if (m_pGraf->ViditelnostKlbu(i)) {
			/* toto bolo stare umiestnovanie ceduliek s nazvami klbov
			int j = 0;
			while (j < VelkostKlbu(i) && !m_pGraf->vGrafe(CPoint(ZistiPozX(j / double(m_nFPS)), ZistiPozY(BodPriebehuKlbu(i, j)))))
				j++;
			if (j < VelkostKlbu(i)) {
				pDC->SetTextColor(m_pGraf->Farba(i));
				pDC->TextOut(5, ZistiPozY(BodPriebehuKlbu(i, j)) - 5, m_pGraf->Nazov(i));
			}
			*/
			//rozdelenie miesta pozdlz vertikalnej osi grafu na POCET_UDAJOV usekov
			const int posun = (m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O)) / POCET_UDAJOV;
			pDC->SetTextColor(m_pGraf->Farba(i));
			pDC->TextOut(5, m_pGraf->Okraj(HORNY_O)+i*posun, m_pGraf->Nazov(i));
		}
}


// nastavenie popisov pre vykreslovane priebehy UR
void CChodza::NastavMenovkyUR(CDC* pDC)
{
	for (int i = 0; i < POCET_UR; i++)
		if (m_pGraf->ViditelnostUR(i)) {
			/* toto bolo stare umiestnovanie ceduliek s nazvami klbov
			int j=0;
			while (j < VelkostUR(i) && !m_pGraf->vGrafe(CPoint(ZistiPozX(double(j + 0.5) / m_nFPS), ZistiPozUR(BodPriebehuUR(i, j)))))
				j++;
			if (j < VelkostUR(i)) {
				pDC->SetTextColor(m_pGraf->Farba(POCET_KLBOV + i));
				pDC->TextOut(5, ZistiPozUR(BodPriebehuUR(i, j)) - 5, m_pGraf->Nazov(POCET_KLBOV + i));
			}
			*/
			//rozdelenie miesta pozdlz vertikalnej osi grafu na POCET_UDAJOV usekov
			const int posun = (m_pGraf->Okraj(DOLNY_O) - m_pGraf->Okraj(HORNY_O)) / POCET_UDAJOV;
			pDC->SetTextColor(m_pGraf->Farba(POCET_KLBOV + i));
			pDC->TextOut(5, m_pGraf->Okraj(HORNY_O)+POCET_KLBOV*posun+i*posun, m_pGraf->Nazov(POCET_KLBOV + i));
		}
}


// vykreslenie vsetkych poloh klbov a hodnot UR
void CChodza::VykresliChodzu(CDC* pDC, int nPoradie)
{
	for (int i = 0; i < POCET_KLBOV; i++) 
		if (m_pGraf->ViditelnostKlbu(i))
			VykresliKlb(i, pDC, nPoradie);
	
	for (int i = 0; i < POCET_UR; i++) 
		if (m_pGraf->ViditelnostUR(i))
			VykresliUR(i, pDC, nPoradie);
}


// prepnutie typu ciary (poradie) a farby pre zadany klb  a jeho vykreslenie
void CChodza::VykresliKlb(int nKlb, CDC* pDC, int nPoradie)
{
	CPen* pPero = new CPen(nPoradie, 1, m_pGraf->Farba(nKlb));
	CPen* pStarePero = pDC->SelectObject(pPero);
	VykresliBodyKlbov(nKlb, pDC);
	pDC->SelectObject(pStarePero);
	delete pPero;
}


// vykreslenie zadaneho klbu
void CChodza::VykresliBodyKlbov(int nKlb, CDC* pDC)
{
	if (VelkostKlbu(nKlb) < 1) return;

	CPoint point(ZistiPozX(0), ZistiPozY(BodPriebehuKlbu(nKlb, 0)));
	pDC->MoveTo(point);
	bool bInGraph = m_pGraf->vGrafe(point);

	for (int j = 1; j < VelkostKlbu(nKlb); j++) {
		point.SetPoint(ZistiPozX(j / double(m_nFPS)), ZistiPozY(BodPriebehuKlbu(nKlb, j)));
		// vykreslujeme len ak sa aktualny a aj predchadzajuci bod nachadza v grafe
		if (bInGraph && m_pGraf->vGrafe(point))
			pDC->LineTo(point);
		else
			pDC->MoveTo(point);
		bInGraph = m_pGraf->vGrafe(point);
	}
}


// prepnutie typu ciary (poradie) a farby pre zadanu UR a jej vykreslenie
void CChodza::VykresliUR(int nUR, CDC* pDC, int nPoradie)
{
	CPen* pPero = new CPen(nPoradie, 1, m_pGraf->Farba(POCET_KLBOV + nUR));
	CPen* pStarePero = pDC->SelectObject(pPero);
	VykresliBodyUR(nUR, pDC);
	pDC->SelectObject(pStarePero);
	delete pPero;
}


// vykreslenie zadanej UR
void CChodza::VykresliBodyUR(int nUR, CDC* pDC)
{
	if (VelkostUR(nUR) < 1) return;

	CPoint point(ZistiPozX(0.5 / m_nFPS), ZistiPozUR(BodPriebehuUR(nUR, 0)));
	pDC->MoveTo(point);
	bool bInGraph = m_pGraf->vGrafe(point);

	for (int j = 1; j < VelkostUR(nUR); j++){
		point.SetPoint(ZistiPozX((j + 0.5) / m_nFPS), ZistiPozUR(BodPriebehuUR(nUR, j)));
		//vykreslujeme len ak sa aktualny a aj predchadzajuci bod nachadza v grafe
		if (bInGraph && m_pGraf->vGrafe(point))
			pDC->LineTo(point);
		else pDC->MoveTo(point);
		bInGraph = m_pGraf->vGrafe(point);
	}
}


// priblizenie nahladu na priebeh v grafe podla konst ZVACSENIE
// na zadany stred pre vsetky okraje
void CChodza::Pribliz(CPoint pt)
{
	const double lavy = HodnotaOkraju(LAVY_O);
	const double pravy = HodnotaOkraju(PRAVY_O);
	const double horny = HodnotaOkraju(HORNY_O);
	const double dolny = HodnotaOkraju(DOLNY_O);

	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x) - (pravy-lavy)/ZVACSENIE;
	double novy_pravy = ZistiHodnotuX(pt.x) + (pravy-lavy)/ZVACSENIE;
	double novy_horny = ZistiHodnotuY(pt.y) - (dolny-horny)/ZVACSENIE;
	double novy_dolny = ZistiHodnotuY(pt.y) + (dolny-horny)/ZVACSENIE;

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
	NastavHodnotyOkrajov(HORNY_O, novy_horny);
	NastavHodnotyOkrajov(DOLNY_O, novy_dolny);
}


// priblizenie nahladu na priebeh v grafe na rozsah
// podla zadanych rohov pre vsetky okraje
void CChodza::Pribliz(CPoint pt1, CPoint pt2)
{
	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy=ZistiHodnotuX(min(pt1.x, pt2.x));
	double novy_pravy=ZistiHodnotuX(max(pt1.x, pt2.x));
	double novy_horny=ZistiHodnotuY(min(pt1.y, pt2.y));
	double novy_dolny=ZistiHodnotuY(max(pt1.y, pt2.y));

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
	NastavHodnotyOkrajov(HORNY_O, novy_horny);
	NastavHodnotyOkrajov(DOLNY_O, novy_dolny);
}


// priblizenie nahladu na priebeh v grafe podla konst ZVACSENIE
// na zadany stred pre lavy a pravy okraj
void CChodza::PriblizLenCas(CPoint pt)
{
	const double lavy = HodnotaOkraju(LAVY_O);
	const double pravy = HodnotaOkraju(PRAVY_O);

	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x) - (pravy-lavy)/ZVACSENIE;
	double novy_pravy = ZistiHodnotuX(pt.x) + (pravy-lavy)/ZVACSENIE;

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
}


// priblizenie nahladu na priebeh v grafe na rozsah podla zadanych rohov pre lavy a pravy okraj
void CChodza::PriblizLenCas(CPoint pt1, CPoint pt2)
{
	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(min(pt1.x, pt2.x));
	double novy_pravy = ZistiHodnotuX(max(pt1.x, pt2.x));

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
}


// vzdialenie nahladu na priebeh v grafe podla konst ZMENSENIE
// na zadany stred pre vsetky okraje
void CChodza::Vzdial(CPoint pt)
{
	const double lavy = HodnotaOkraju(LAVY_O);
	const double pravy = HodnotaOkraju(PRAVY_O);
	const double horny = HodnotaOkraju(HORNY_O);
	const double dolny = HodnotaOkraju(DOLNY_O);

	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x) - (pravy-lavy)/ZMENSENIE;
	double novy_pravy = ZistiHodnotuX(pt.x) + (pravy-lavy)/ZMENSENIE;
	double novy_horny = ZistiHodnotuY(pt.y) - (dolny-horny)/ZMENSENIE;
	double novy_dolny = ZistiHodnotuY(pt.y) + (dolny-horny)/ZMENSENIE;

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
	NastavHodnotyOkrajov(HORNY_O, novy_horny);
	NastavHodnotyOkrajov(DOLNY_O, novy_dolny);
}


// vzdialenie nahladu na priebeh v grafe na rozsah
// podla zadanych rohov pre lavy a pravy okraj
void CChodza::VzdialLenCas(CPoint pt)
{
	const double lavy = HodnotaOkraju(LAVY_O);
	const double pravy = HodnotaOkraju(PRAVY_O);

	//nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	//hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x) - (pravy-lavy)/ZMENSENIE;
	double novy_pravy = ZistiHodnotuX(pt.x) + (pravy-lavy)/ZMENSENIE;

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
}


// posun stredu nahladu na priebeh v grafe podla zadaneho posunu pre vsetky okraje
void CChodza::Stred(CPoint pt)
{
	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x+m_pGraf->Okraj(LAVY_O));
	double novy_pravy = ZistiHodnotuX(pt.x+m_pGraf->Okraj(PRAVY_O));
	double novy_horny = ZistiHodnotuY(pt.y+m_pGraf->Okraj(HORNY_O));
	double novy_dolny = ZistiHodnotuY(pt.y+m_pGraf->Okraj(DOLNY_O));

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
	NastavHodnotyOkrajov(HORNY_O, novy_horny);
	NastavHodnotyOkrajov(DOLNY_O, novy_dolny);
}


// posun stredu nahladu na priebeh v grafe
// podla zadaneho posunu pre lavy a pravy okraj
void CChodza::StredLenCas(CPoint pt)
{
	// nie je mozne hned priamo nastavovat hodnoty okrajov, lebo ich povodna 
	// hodnota by sa prepisala a ovplyvnila dalsie vypocty
	double novy_lavy = ZistiHodnotuX(pt.x + m_pGraf->Okraj(LAVY_O));
	double novy_pravy = ZistiHodnotuX(pt.x + m_pGraf->Okraj(PRAVY_O));

	NastavHodnotyOkrajov(LAVY_O, novy_lavy);
	NastavHodnotyOkrajov(PRAVY_O, novy_pravy);
}