//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				Chodza.h
// Ucel suboru:			deklaracia triedy CChodza - trieda uchovava udaje o nacitanej chodzi, 
//						polohy klbov a hodnoty UR, a hodnoty okrajov pri vykreslovani do grafu
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Martin Soltis
// Naposledy zmenil:	Soltis / 20.4.2005
// Poznamka:			UR = uhlova rychlost
//______________________________________________________________________________________________________


#pragma once

#include "udaje.h"
#include "graf.h"

// hodnoty okrajov grafu
#define POCET_HODNOT_OKRAJOV	6
// zvacsenie nahladu pri praci s priebehmi 125%
#define ZVACSENIE				(1.25*2)
// zmensenie nahladu pri praci s priebehmi 80%
#define ZMENSENIE				(0.8*2)

// trieda CChodza uchovava udaje o nacitanej chodzi,
// polohy klbov a hodnoty UR, a hodnoty okrajov pri vykreslovani do grafu
class CChodza : public CUdaje
{
	// okraje zobrazovanych priebehov
	double	m_fHodnotyOkrajov[POCET_HODNOT_OKRAJOV];

	CString	m_strMeno;			// m_meno vstupneho suboru
	int		m_nFPS;				// pocet frameov za sekundu
	CGraf*	m_pGraf;			// graf v ktorom sa priebeh vykresluje

public:
	CChodza(CGraf* pGraf, CString strMeno, int nFPS);
	~CChodza(void);

	// praca s hodnotami okrajov
	void NastavHodnotyOkrajov(int nOkraj, double fHodnota);
	void NastavHodnotyOkrajov(double fHorny, double fDolny, double fLavy, double fPravy, double fHornyUR, double fDolnyUR);
	double HodnotaOkraju(int nOkraj);

	// automaticke nastavovenie okrajov tak, aby sa cely priebeh
	// nachadzal medzi hodnotami okrajov
	void NastavHodnotyOkrajov();
	void NastavHodnotyHorizontalnychOkrajov();
	void NastavHodnotyOkrajovUR();
	void NastavHodnotyCasovychOkrajov();

	// ziskanie clenskych premennych
	CString GetMeno(void) const { return m_strMeno; }	// vracia meno vstupneho suboru priebehov
	int GetFPS(void) const { return m_nFPS; }			// vracia pocet snimkov za sekundu nacitanych priebehov
	CGraf* GetGraf(void) const { return m_pGraf; }		// vracia graf, v ktorom su polohy klbov a hodnoty UR vykreslovane

	// prepocet pozicie v grafe na hodnotu v grafe
	double ZistiHodnotuX(int nPozX) const;
	double ZistiHodnotuY(int nPozY) const;
	double ZistiHodnotuUR(int nPozY) const;

	// prepocet hodnoty v grafe na poziciu v grafe
	int ZistiPozX(double fHodX) const;
	int ZistiPozY(double fHodY) const;
	int ZistiPozUR(double fHodY) const;

	// ziskanie suradnic (hodnot) zadaneho bodu v grafe 
	CString GetSuradnice(CPoint pt, CString strOs) const;

	// nastavenie pomocnych popisov pre vykreslovane polohy klbov a hodnoty UR
	void NastavMenovky(CDC* pDC);
	void NastavMenovkyKlbom(CDC* pDC);
	void NastavMenovkyUR(CDC* pDC);

	// vykreslovanie poloh klbov a hodnot UR
	void VykresliChodzu(CDC* pDC, int nPoradie);
	void VykresliKlb(int nKlb, CDC* pDC, int nPoradie);
	void VykresliBodyKlbov(int nKlb, CDC* pDC);
	void VykresliUR(int nUR, CDC* pDC, int nPoradie);
	void VykresliBodyUR(int nKlb, CDC* pDC);

	// priblizovanie, vzdalovanie a posuvanie sa v grafe
	void Pribliz(CPoint pt);
	void Pribliz(CPoint pt1, CPoint pt2);
	void PriblizLenCas(CPoint pt);
	void PriblizLenCas(CPoint pt1, CPoint pt2);
	void Vzdial(CPoint pt);
	void VzdialLenCas(CPoint pt);
	void Stred(CPoint pt);
	void StredLenCas(CPoint pt);
};
