//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				Graf.cpp
// Ucel suboru:			definicia triedy CGraf - trieda uchovava informacie (viditelnost priebehehov, 
//						pozicia grafu, popis grafu) o grafe, v ktorom su vykreslovane priebehy hodnot
//						pozicii klbov a hodnot UR
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Martin Soltis
// Naposledy zmenil:	Lucenic / 5.5.2005
// Poznamka:			UR = uhlova rychlost
//______________________________________________________________________________________________________


#include "stdafx.h"
#include "graf.h"


CGraf::CGraf(void)
{
	m_strPopisVertOsi = _T("");
	m_strPopisGrafu = _T("");
	m_strPopisHorizOsi = _T("");

	NaplnFarby();
	NastavNazvy();
}


CGraf::~CGraf(void)
{
}


// pociatocne nastavenie viditelnosti klbov a UR
void CGraf::NastavViditelnost()
{
	for (int i = 0; i < POCET_KLBOV; i++) {
		if (i == L_BEDROVY || i == P_BEDROVY)
			m_bViditelnost[i] = false;
		else
			m_bViditelnost[i] = true;
	}
	for (int i = POCET_KLBOV; i < POCET_UDAJOV; i++)
		m_bViditelnost[i] = false;
}


// nastavenie viditelnosti zadaneho klbu
void CGraf::NastavViditelnostKlbu(int nKlb, bool bVidno)
{
	ASSERT(nKlb > -1 && nKlb < POCET_KLBOV);
	m_bViditelnost[nKlb] = bVidno;
}


// nastavenie viditelnosti zadanej UR
void CGraf::NastavViditelnostUR(int nUR, bool bVidno)
{
	ASSERT(nUR > -1 && nUR < POCET_UR);
	m_bViditelnost[nUR + POCET_KLBOV] = bVidno;
}


// zmena viditelnosti zadaneho klbu
void CGraf::ZmenViditelnostKlbu(int nKlb)
{
	ASSERT(nKlb > -1 && nKlb < POCET_KLBOV);
	m_bViditelnost[nKlb] = !m_bViditelnost[nKlb];
}


// zmena viditelnosti zadanej UR
void CGraf::ZmenViditelnostUR(int nUR)
{
	ASSERT(nUR > -1 && nUR < POCET_UR);
	m_bViditelnost[nUR + POCET_KLBOV] = !m_bViditelnost[nUR + POCET_KLBOV];
}


// vracia ci bude zadany klb v grafe vykreslovany
bool CGraf::ViditelnostKlbu(int nKlb)
{
	ASSERT(nKlb > -1 && nKlb < POCET_KLBOV);
	return m_bViditelnost[nKlb];
}


// vracia ci bude zadana UR v grafe vykreslovana 
bool CGraf::ViditelnostUR(int nUR)
{
	//ASSERT(nUR>-1 && nUR<POCET_UR);
	return m_bViditelnost[nUR + POCET_KLBOV];
}


// nastavenie pozicie zadaneho okraju grafu
void CGraf::NastavOkraj(int nOkraj, int nHodnota)
{
	ASSERT(nOkraj > -1 && nOkraj < POCET_OKRAJOV);
	m_nOkraj[nOkraj] = nHodnota;
}


// nastavenie pozicie okrajov a osi grafu
void CGraf::NastavOkraje(int nHorny, int nDolny, int nLavy, int nPravy, int nOs)
{
	m_nOkraj[HORNY_O] = nHorny;
	m_nOkraj[DOLNY_O] = nDolny;
	m_nOkraj[LAVY_O] = nLavy;
	m_nOkraj[PRAVY_O] = nPravy;
	m_nPozCasOsi = nOs;
}


// vracia poziciu zadaneho okraju grafu
int CGraf::Okraj(int nOkraj)
{
	ASSERT(nOkraj > -1 && nOkraj < POCET_OKRAJOV);
	return m_nOkraj[nOkraj];
}


// vracia poziciu osi grafu
int CGraf::GetOs() 
{
	return m_nPozCasOsi;
}


// nastavenie popisu osi grafu a nazvu grafu
void CGraf::NastavPopis(CString str1, CString str2, CString str3)
{
	m_strPopisVertOsi = str1;
	m_strPopisGrafu = str2;
	m_strPopisHorizOsi = str3;
}


// vykreslenie osi grafu, ich popisu a nazvu
void CGraf::NakresliOsi(CDC* pDC)
{
	//osi grafu
	pDC->MoveTo(m_nOkraj[LAVY_O], m_nOkraj[HORNY_O] - 5);
	pDC->LineTo(m_nOkraj[LAVY_O], m_nOkraj[DOLNY_O] + 5);
	pDC->MoveTo(m_nOkraj[LAVY_O] - 5, m_nPozCasOsi);
	pDC->LineTo(m_nOkraj[PRAVY_O] + 5, m_nPozCasOsi);
	
	//popisy grafu
	pDC->SetBkColor(RGB(255, 255, 255));
	pDC->TextOut(5, m_nOkraj[HORNY_O] - 30, m_strPopisVertOsi);
	pDC->TextOut(5, m_nOkraj[HORNY_O] - 15, m_strPopisGrafu);
	pDC->TextOut(m_nOkraj[PRAVY_O] - 15, m_nPozCasOsi + 5, m_strPopisHorizOsi);
}


// vracia true ak sa zadany bod nachadza nad oblastou grafu
// (plus OKRAJ_OKOLO_GRAFU pixelov na kazdu stranu)
bool CGraf::vGrafe(CPoint pt)
{
	return (m_nOkraj[LAVY_O] - OKRAJ_OKOLO_GRAFU <= pt.x && 
		pt.x <= m_nOkraj[PRAVY_O] + OKRAJ_OKOLO_GRAFU &&
		m_nOkraj[HORNY_O] - OKRAJ_OKOLO_GRAFU <= pt.y && 
		pt.y <= m_nOkraj[DOLNY_O] + OKRAJ_OKOLO_GRAFU);
}


// vracia zadanu farbu (farbu vykreslovaneho priebehu)
COLORREF CGraf::Farba(int nFarba)
{
	ASSERT(nFarba > -1 && nFarba < POCET_FARIEB);
	return m_crFarba[nFarba];
}


// vracia zadany nazov (nazov vykreslovaneho priebehu)
CString CGraf::Nazov(int nNazov)
{
	ASSERT(nNazov > -1 && nNazov < POCET_UDAJOV);
	return m_strNazovUdaju[nNazov];
}


// naplnenie farieb pre vykreslovane priebehy do pola
void CGraf::NaplnFarby(void)
{
	m_crFarba[0] = RGB(255, 0, 0);
	m_crFarba[1] = RGB(0, 0, 255);
	m_crFarba[2] = RGB(128, 0, 0);
	m_crFarba[3] = RGB(0, 128, 0);
	m_crFarba[4] = RGB(0, 0, 128);
	m_crFarba[5] = RGB(128, 128, 0);
	m_crFarba[6] = RGB(128, 0, 128);
	m_crFarba[7] = RGB(0, 128, 128);
	m_crFarba[8] = RGB(255, 0, 255);
	m_crFarba[9] = RGB(255, 128, 0);
	m_crFarba[10] = RGB(128, 0, 255);
	m_crFarba[11] = RGB(0, 128, 255);
	m_crFarba[12] = RGB(255, 0, 128);
	m_crFarba[13] = RGB(128, 128, 128);
	m_crFarba[14] = RGB(0, 0, 0);
	m_crFarba[15] = RGB(255, 255, 255);
}


// naplnenie nazvov popisov pre vykreslovane priebehy do pola
void CGraf::NastavNazvy(void)
{
	m_strNazovUdaju[0].Format("L pelvis");
	m_strNazovUdaju[1].Format("L knee");
	m_strNazovUdaju[2].Format("L ankle");
	m_strNazovUdaju[3].Format("L heel");
	m_strNazovUdaju[4].Format("L toes");
	m_strNazovUdaju[5].Format("R pelvis");
	m_strNazovUdaju[6].Format("R knee");
	m_strNazovUdaju[7].Format("R ankle");
	m_strNazovUdaju[8].Format("R heel");
	m_strNazovUdaju[9].Format("R toes");
	m_strNazovUdaju[10].Format("LAV k-a-toes");
	m_strNazovUdaju[11].Format("LAV k-a-heel");
	m_strNazovUdaju[12].Format("RAV k-a-toes");
	m_strNazovUdaju[13].Format("RAV k-a-heel");
}