// Animation.cpp: implementation of the CAnimation class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "bluebone.h"
#include "blueboneDlg.h"
#include "Animation.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAnimation::CAnimation()
{ 
  m_FPS           = 30;
  m_loops_count   = 1;
  m_loop_startF   = 0;
  m_loop_endF     = 0;
  m_frames_count  = 0;
  m_current_loop  = 0;
  m_current_frame = 0;
  m_fileName = new char[50];
  sprintf(m_fileName, "new animation");
  m_bPaused = FALSE;
}

CAnimation::CAnimation(const char* acFileName)
{
  //zaciatocne uhly
  m_lastAngles[ID_PELVIS] = 0;
  m_lastAngles[ID_L_FEMUR] = m_lastAngles[ID_R_FEMUR] = 90;
  m_lastAngles[ID_L_LOWER_LEG] = m_lastAngles[ID_R_LOWER_LEG] = 180;
  m_lastAngles[ID_L_FOOT] = m_lastAngles[ID_R_FOOT] = -110;
  m_lastAngles[ID_L_TOE1] = m_lastAngles[ID_L_TOE2] = m_lastAngles[ID_L_TOE3] = m_lastAngles[ID_L_TOE4] = m_lastAngles[ID_L_TOE5] = 180;
  m_lastAngles[ID_R_TOE1] = m_lastAngles[ID_R_TOE2] = m_lastAngles[ID_R_TOE3] = m_lastAngles[ID_R_TOE4] = m_lastAngles[ID_R_TOE5] = 180;

	std::ifstream input(acFileName);
	//read in the frames
  SVertex vertex;
  input >> m_FPS;
	input >> m_frames_count;
  input >> m_loops_count;
  for (int i = 0; i < m_frames_count; i++) {
    for (int j=0; j<VERTICES_PER_FRAME; j++) {
      ZeroMemory(&vertex, sizeof(SVertex));
      input >> vertex.x;
      input >> vertex.y;
      input >> vertex.z;
      m_framesRead.push_back(vertex);
    }
	}

  std::vector<SNodeMov> frames;
  SNodeMov frame;
  for (i = 0; i < m_frames_count; i++) {
    frames.clear();
    for (int j = ID_PELVIS; j <= ID_R_TOE5; j++) {
	    ZeroMemory(&frame, sizeof(SNodeMov));
      frame.nodeID = j;
      if (j == ID_PELVIS) frame.axisID = 1;
      else frame.axisID = 0;
      frame.angle = ComputeAngle(j, i);
      if (frame.angle != 0) frames.push_back(frame); 
    }
    m_frames.push_back(frames);
  }
  m_loop_startF = 1;
  m_loop_endF    = m_frames.size();
  m_current_loop = 0;
  m_current_frame = 0;
  m_fileName = new char[50];
  sprintf(m_fileName, "new animation");
  m_bPaused = FALSE;
}

CAnimation::~CAnimation()
{
	/*for (int i=1; i<=ID_LAST_NODE; i++)
	{
		CNode* pkNode = m_rootNode->FindNode(i);
		pkNode->RemoveAxes();
		pkNode->ResetRotation();
	}*/
  ((CBlueBoneDlg*)AfxGetMainWnd())->KillTimer(IDT_TIMER);
	m_framesRead.clear();
	for (int i=0; i<m_frames.size(); i++) m_frames[i].clear();
  m_frames.clear();
  if (m_fileName)
  {
    delete [] m_fileName;
    m_fileName = NULL;
  }
}

double CAnimation::ComputeAngle(unsigned int nodeID, unsigned int frame)
{
  CVector3 vector1, vector2, vector_cross;
  int i,j,k;
  double lastAngle, angle;
  switch (nodeID) {
    case ID_PELVIS:       i=5; j=0; break;
    case ID_L_FEMUR:      i=5; j=6; break;
    case ID_R_FEMUR:      i=0; j=1; break;
    case ID_L_LOWER_LEG:  i=6; j=7; k=5; break;
    case ID_R_LOWER_LEG:  i=1; j=2; k=0; break;
    case ID_L_FOOT:       i=7; j=8; k=6; break;
    case ID_R_FOOT:       i=2; j=3; k=1; break;
    case ID_L_TOE1:
    case ID_L_TOE2:
    case ID_L_TOE3:
    case ID_L_TOE4:
    case ID_L_TOE5:       i=8; j=9; k=7; break;
    case ID_R_TOE1:
    case ID_R_TOE2:
    case ID_R_TOE3:
    case ID_R_TOE4:
    case ID_R_TOE5:       i=3; j=4; k=2; break;
    default:              return 0;
  }
  lastAngle = m_lastAngles[nodeID]; //predchadzajuci stav uhla
  vector1.FromVertices(m_framesRead[frame*VERTICES_PER_FRAME+i], m_framesRead[frame*VERTICES_PER_FRAME+j]);
  if (nodeID == ID_PELVIS) {
    vector2.X() = 0;
    vector2.Y() = 0;
    vector2.Z() = 1;
  }
  else if (nodeID == ID_L_FEMUR || nodeID == ID_R_FEMUR) {
    vector2.X() = 1;
    vector2.Y() = 0;
    vector2.Z() = 0;
  }
  else {
    vector2.FromVertices(m_framesRead[frame*VERTICES_PER_FRAME+i], m_framesRead[frame*VERTICES_PER_FRAME+k]);
  }
  angle = vector1.Angle(vector2);
  vector_cross = vector1.Cross(vector2);
  if (vector_cross.Z() > 0) angle = -angle;
  m_lastAngles[nodeID] = angle;
  if (lastAngle - angle > 180) return lastAngle - angle - 360;
  else return lastAngle - angle;
}

void CAnimation::OnTimer(CNode *rootNode)
{
  if (m_current_loop == 0) m_current_loop = 1;

	if (m_frames.size() == 0) {
		Stop();
		return;
	}

	for (unsigned int i = 0; i < m_frames[m_current_frame].size(); i++)
  {
		CNode* node = rootNode->FindNode(m_frames[m_current_frame][i].nodeID);
    
		if (node)
			node->RotateAroundAxis(m_frames[m_current_frame][i].angle, m_frames[m_current_frame][i].axisID);
	}

  m_current_frame++;

  if (m_current_frame == m_frames.size()) {
    if (m_current_loop < m_loops_count || m_loops_count == -1) {
      m_current_frame = m_loop_startF;
      m_current_loop++;
    }
    else {
		  Stop();
    }
	}
}


void CAnimation::Play()
{
  CMenu *menu = ((CBlueBoneDlg*)AfxGetMainWnd())->GetMenu();
  menu->EnableMenuItem(ID_ANIMATION_PLAY, MF_GRAYED);
  menu->EnableMenuItem(ID_ANIMATION_STOP, MF_ENABLED);
  menu->EnableMenuItem(ID_ANIMATION_PAUSE, MF_ENABLED);
  
  // ak to bolo pauznute, tak to odpauzujeme
  if (m_bPaused)
  {
    m_bPaused = FALSE;
    menu->ModifyMenu(ID_ANIMATION_PAUSE, MF_STRING, ID_ANIMATION_PAUSE, "Pa&use");
  }
  // inak nastavime aktualny frejm a loop na zaciatok
  else
  {
    m_current_frame = 0;
    m_current_loop = 0;
  }

  ((CBlueBoneDlg*)AfxGetMainWnd())->DrawMenuBar();

  ((CBlueBoneDlg*)AfxGetMainWnd())->SetTimer(IDT_TIMER, 1000/m_FPS, NULL);
}

void CAnimation::Pause()
{
  CMenu *menu = ((CBlueBoneDlg*)AfxGetMainWnd())->GetMenu();
  if (!m_bPaused)
  {
    m_bPaused = TRUE;
    ((CBlueBoneDlg*)AfxGetMainWnd())->KillTimer(IDT_TIMER);
    menu->ModifyMenu(ID_ANIMATION_PAUSE, MF_STRING, ID_ANIMATION_PAUSE, "&Continue");
    menu->EnableMenuItem(ID_ANIMATION_PLAY, MF_ENABLED);
  }
  else
  {
    menu->ModifyMenu(ID_ANIMATION_PAUSE, MF_STRING, ID_ANIMATION_PAUSE, "Pa&use");
    Play();
  }
}

void CAnimation::Stop()
{
  ((CBlueBoneDlg*)AfxGetMainWnd())->KillTimer(IDT_TIMER);

  m_current_frame = 0;
  m_current_loop = 0;

  CMenu *menu = ((CBlueBoneDlg*)AfxGetMainWnd())->GetMenu();
  menu->EnableMenuItem(ID_ANIMATION_STOP, MF_GRAYED);
  menu->EnableMenuItem(ID_ANIMATION_PAUSE, MF_GRAYED);
  menu->EnableMenuItem(ID_ANIMATION_PLAY, MF_ENABLED);
  ((CBlueBoneDlg*)AfxGetMainWnd())->DrawMenuBar();
}

void CAnimation::Save(const char* acFileName)
{

  delete [] m_fileName;
  m_fileName = new char[strlen(acFileName)+5];
  strcpy(m_fileName, acFileName);

	std::ofstream output(acFileName);

  output << m_FPS ;
  output << '\n';
	output << m_frames.size();
  output << '\n';
	output << m_loops_count;
  output << "\n\n";

	for (int k = 0; k < m_frames.size(); k++)
	{
		output << m_frames[k].size();
    output << '\n';
		for (int l = 0; l < m_frames[k].size(); l++)
		{
			output << m_frames[k][l].nodeID;
      output << ' ';
			output << m_frames[k][l].axisID;
      output << ' ';
			output << m_frames[k][l].angle;
			if (l == m_frames[k].size() - 1)
				output << "\n\n";
			else
				output << ' ';
		}
	}
}


void CAnimation::Load(const char* acFileName)
{
  delete [] m_fileName;
  m_fileName = new char[strlen(acFileName)+5];
  strcpy(m_fileName, acFileName);

  std::ifstream input(acFileName);

  std::vector<SNodeMov> frame;
  SNodeMov node_movement;
  unsigned int nodes_per_frame = 0;
  
  input >> m_FPS;
	input >> m_frames_count;
  input >> m_loops_count;
	
  for (unsigned int k = 0; k < m_frames_count; k++)
	{
    frame.clear();
    nodes_per_frame = 0;
		input >> nodes_per_frame;
		for (unsigned int p = 0; p < nodes_per_frame; p++)
		{
			ZeroMemory(&node_movement, sizeof(SNodeMov));
			input >> node_movement.nodeID;
			input >> node_movement.axisID;
			input >> node_movement.angle;
			frame.push_back(node_movement);
		}
		m_frames.push_back(frame);
	}
  m_loop_startF = 1;
  m_loop_endF = m_frames.size();
}

void CAnimation::TakeKeyFrame(bool bInit)
{
  int max_diff = 0;
  unsigned int i, j;
	if (bInit)
		m_frames_count = 0;

	std::vector<SNodeMov> frame;
	for (i = 1; i <= ID_LAST_NODE; i++)	// cez vsetky uzly
	{
		CNode* pkNode = ((CBlueBoneDlg*)AfxGetMainWnd())->GetRootNode()->FindNode(i);
		if (bInit)
			pkNode->MakeSnapshot();

		std::vector<GLfloat> kOldSnapshot; 
		pkNode->GetSnapshot(kOldSnapshot);

		pkNode->MakeSnapshot();

		std::vector<GLfloat> kNewSnapshot;
		pkNode->GetSnapshot(kNewSnapshot);

		for (j = 0; j < kNewSnapshot.size(); j++)	//skontrolujeme otocenie okol vsetkych osi
    {
      
			if (kOldSnapshot[j] != kNewSnapshot[j])	//ak su rozdielne
			{
				SNodeMov nodemove;
				nodemove.angle = kNewSnapshot[j] - kOldSnapshot[j];
        if ( abs(nodemove.angle) > max_diff ) max_diff = abs(nodemove.angle);
				nodemove.axisID = j;
				nodemove.nodeID = i;
				frame.push_back(nodemove);
			}
    }
	}
  // ak sa kostra ohla v niektorom klbe o prilis velky uhol, musime to rozlozit
  // na viac snimkov
  if (max_diff > MAX_ANGLE_PER_FRAME)
  {
    // vypocitame na kolko frejmov musime rozlozit zachyteny pohyb, aby sme
    // neprekrocili maximalny maximalnu velkost uhla o ktoru sa moze nieco otocit
    // (kvoli plynulosti pohybu)
    int nf = (int) ceil( (double)max_diff / (double)MAX_ANGLE_PER_FRAME );
    // prejdeme cez vsetky uzly vo vytvorenom frejme a vydelime uhol rotacie
    // poctom frejmov, na ktore to chceme rozlozit
    for (i = 0; i < frame.size(); i++)
    {
      frame[i].angle /= (double) nf;
    }
    for (i = 0; i < nf; i++)
    {
      m_frames.push_back(frame);
      m_frames_count++;
    }
  }
  else
  {
    if (frame.size() > 0)
    {
      m_frames.push_back(frame);
      m_frames_count++;
    }
  }
  m_current_frame = m_frames_count - 1;
}

BOOL CAnimation::ShowFrame(unsigned int n)
{
  CNode* rootNode = ((CBlueBoneDlg*)AfxGetMainWnd())->GetRootNode();
  if (n < 0 || n >= m_frames[n].size()) return FALSE;
  for (unsigned int i = 0; i < m_frames[n].size(); i++)
	{
		CNode* node = rootNode->FindNode(m_frames[n][i].nodeID); 
		if (node)
			node->RotateAroundAxis(m_frames[n][i].angle, m_frames[n][i].axisID);
	}
  return TRUE;
}