// Animation.h: interface for the CAnimation class.
//
//////////////////////////////////////////////////////////////////////

#include "Mesh.h"
#include "vector3.h"
#include "node.h"
#include <fstream>


#if !defined(AFX_ANIMATION_H__1D167A6C_73DE_42EB_A6C4_F403A222AC78__INCLUDED_)
#define AFX_ANIMATION_H__1D167A6C_73DE_42EB_A6C4_F403A222AC78__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define VERTICES_PER_FRAME 10

// obsahuje udaje o rotacii jedneho uzla okolo danej osi
// - kazdy snimok (frejm) sa sklada z viacerych SNodeMov -
// podla toho s kolkymi uzlami sa v danom frejme pohlo
struct SNodeMov
{
	unsigned int nodeID; //ktorou kostou sa hybe
	unsigned int axisID; //podla ktorej osi
	GLfloat angle;       //o aky uhol
};


class CAnimation  
{
public:
	void Load(const char* acFileName);
	void Save(const char* acFileName);
	void OnTimer(CNode *rootNode);
	void Play();
  void Pause();
  void Stop();
	void TakeKeyFrame(bool bInit);
  BOOL ShowFrame(unsigned int n);
  unsigned int GetFramesCount() {return m_frames_count;};
  unsigned int GetCurrentFrame() {return m_current_frame;};
  unsigned int GetLoopsCount() {return m_loops_count;};
  unsigned int GetCurrentLoop() {return m_current_loop;};
  unsigned int GetFPS() {return m_FPS;};
  const char* GetFileName() {return m_fileName;};
	double ComputeAngle(unsigned int nodeID, unsigned int frame);
  CAnimation();
	CAnimation(const char* acFileName);
	virtual ~CAnimation();
private:
  //pocet snimkov za sekundu (frames per second)
  //vektor poli vrcholov nacitanych so suboru
  std::vector<SVertex> m_framesRead;
  //podla snimkov sa bude robit animacia (je to v podstate vektor vektorov)
  std::vector< std::vector<SNodeMov> > m_frames;
  //uklada sa vzdy posledny stav uhlov medzi kostami, aby sa vedelo ci sa zmenili
  GLfloat m_lastAngles[ID_LAST_NODE+1];
  
  int m_loops_count;            // pocet opakovani animacie (ak je -1, tak sa stale opakuje)
  unsigned int m_current_loop;  // aktualne opakovanie
  unsigned int m_current_frame; // aktualny frejm animacie
  unsigned int m_loop_startF;   // zaciatocny frejm slucky (loop start) 
  unsigned int m_loop_endF;     // koncovy frejm slucky (loop end)
  unsigned int m_FPS;           // frejmy za sekundu
  unsigned int m_frames_count;  // celkovy pocet frejmov animacie
  char* m_fileName;
  BOOL m_bPaused;
};

#endif // !defined(AFX_ANIMATION_H__1D167A6C_73DE_42EB_A6C4_F403A222AC78__INCLUDED_)
