#include "StdAfx.h"
#include "camera.h"
#include "Matrix3.h"

CCamera::CCamera() 
	: m_kUp(CVector3::UNIT_Y), m_kLeft(CVector3::UNIT_X*(-1)), m_kLocation(CVector3::ZERO),
	m_kDirection(CVector3::UNIT_Z*(-1)), m_kArcCenter(CVector3(0, 0, 0))
{/*****/}
	
CCamera::CCamera (CVector3 kUp, CVector3 kLeft, CVector3 kLocation, CVector3 kDirection) 
	: m_kUp(kUp), m_kLeft(kLeft), m_kLocation(kLocation), m_kDirection(kDirection), 
	m_kArcCenter(CVector3(0, 0, 0))
{/*****/}


void CCamera::SetViewMatrix ()
/* UCEL:   nastavenie MODELVIEW matice OpenGL
   VSTUP:  -
   VYSTUP: -
*/
{
	glMatrixMode (GL_MODELVIEW);
	CVector3 kCenter = m_kLocation + m_kDirection;
	gluLookAt (m_kLocation.X (), m_kLocation.Y (), m_kLocation.Z (),
		kCenter.X (), kCenter.Y (), kCenter.Z (), m_kUp.X (), m_kUp.Y (), m_kUp.Z ());

}


void CCamera::TurnLeft (double dQuantum)
/* UCEL:   otocenie pohladu dolava
   VSTUP:  dQuantum - kvantifikacia otocenia
   VYSTUP: -
*/
{
	CMatrix3 kRotationMatrix = CMatrix3::ZERO;
	kRotationMatrix.FromAngleAxis (m_kUp, dQuantum);	
	m_kLeft = kRotationMatrix * m_kLeft;			
	m_kDirection = kRotationMatrix * m_kDirection;	
}


void CCamera::LookDown (double dQuantum)
/* UCEL:   otocenie pohladu nadol
   VSTUP:  dQuantum - kvantifikacia otocenia
   VYSTUP: -
*/
{
	CMatrix3 kRotationMatrix = CMatrix3::ZERO;
	kRotationMatrix.FromAngleAxis (m_kLeft, dQuantum);
	m_kUp = kRotationMatrix * m_kUp;
	m_kDirection = kRotationMatrix * m_kDirection;
}


void CCamera::ArcX (double dQuantum)
/* UCEL:   orbit kamery v x-ovej rovine
   VSTUP:  dQuantum - kvantifikacia orbitu
   VYSTUP: -
*/
{
	CVector3 kRadius (m_kLocation - m_kArcCenter);
	CMatrix3 kRotationMatrix (CMatrix3::IDENTITY);
	// kRotationMatrix.FromAngleAxis (m_kUp, dQuantum); // volna rotacia
  kRotationMatrix.FromAngleAxis (CVector3::UNIT_Y, dQuantum);  // rotacia podla osi y
	CVector3 kNewRadius = kRotationMatrix * kRadius;
	
  m_kLocation += (kNewRadius - kRadius);	//posun kamery po kruznici
	m_kLeft = kRotationMatrix * m_kLeft;		//a otocenie smerovych vektorov
  m_kLeft.Normalize();
	m_kDirection = kRotationMatrix * m_kDirection;
	m_kUp = kRotationMatrix * m_kUp;
  m_kUp.Normalize();
}


void CCamera::ArcY (double dQuantum)
/* UCEL:   orbit kamery v y-ovej rovine
   VSTUP:  dQuantum - kvantifikacia orbitu
   VYSTUP: -
*/
{
  
  CVector3 kRadius (m_kLocation - m_kArcCenter);
	CMatrix3 kRotationMatrix (CMatrix3::IDENTITY);
	kRotationMatrix.FromAngleAxis (m_kLeft, -dQuantum);
	CVector3 kNewRadius = kRotationMatrix * kRadius;

  m_kLocation += (kNewRadius - kRadius);		//posun kamery po kruznici
 	m_kLeft = kRotationMatrix * m_kLeft;		  //a otocenie smerovych vektorov;
  m_kLeft.Normalize();
	m_kDirection = kRotationMatrix * m_kDirection;
	m_kUp = kRotationMatrix * m_kUp;
  m_kUp.Normalize();
}