#include "StdAfx.h"
#include "matrix3.h"
#include "math.h"

const CMatrix3 CMatrix3::IDENTITY (1, 0, 0, 0, 1, 0, 0, 0, 1);
const CMatrix3 CMatrix3::ZERO (0, 0, 0, 0, 0, 0, 0, 0, 0);

CMatrix3::CMatrix3 (const CVector3 kRow1, const CVector3 kRow2, const CVector3 kRow3)
/* UCEL:   konstruktor
   VSTUP:  kRow1 - vektor reprezentujuci 1. riadok matice
	   kRow2 - vektor reprezentujuci 2. riadok matice
 	   kRow3 - vektor reprezentujuci 3. riadok matice
   VYSTUP: -
*/
{
	m_aadItem [0][0] = kRow1 [0];
	m_aadItem [0][1] = kRow1 [1];
	m_aadItem [0][2] = kRow1 [2];

	m_aadItem [1][0] = kRow2 [0];
	m_aadItem [1][1] = kRow2 [1];
	m_aadItem [1][2] = kRow2 [2];

	m_aadItem [2][0] = kRow3 [0];
	m_aadItem [2][1] = kRow3 [1];
	m_aadItem [2][2] = kRow3 [2];
}


CMatrix3::CMatrix3 (const double d11, const double d12, const double d13,
			const double d21, const double d22, const double d23, 
			const double d31, const double d32, const double d33)
/* UCEL:   konstruktor
   VSTUP:  dij - prvok matice na pozicii [i,j]
   VYSTUP: -
*/
{
	m_aadItem [0][0] = d11;
	m_aadItem [0][1] = d12;
	m_aadItem [0][2] = d13;
	m_aadItem [1][0] = d21;
	m_aadItem [1][1] = d22;
	m_aadItem [1][2] = d23;
	m_aadItem [2][0] = d31;
	m_aadItem [2][1] = d32;
	m_aadItem [2][2] = d33;
}


CMatrix3::CMatrix3 (const CMatrix3& rkMatrix)
/* UCEL:   kopirovaci konstruktor
   VSTUP:  rkMatrix - referencna matica na kopirovanie
   VYSTUP: -
*/
{
	memcpy (m_aadItem, rkMatrix.m_aadItem, 9*sizeof(double));
}


CMatrix3::CMatrix3 (double aadNew [16])
/* UCEL:   konstruktor
   VSTUP:  aadNew - pole prvok matice vo formate OpenGL!
   VYSTUP: -
*/
{
	m_aadItem [0][0] = aadNew [0];
	m_aadItem [0][1] = aadNew [4];
	m_aadItem [0][2] = aadNew [8];
	m_aadItem [1][0] = aadNew [1];
	m_aadItem [1][1] = aadNew [5];
	m_aadItem [1][2] = aadNew [9];
	m_aadItem [2][0] = aadNew [2];
	m_aadItem [2][1] = aadNew [6];
	m_aadItem [2][2] = aadNew [10];
}


void CMatrix3::Transpose ()
/* UCEL:   vypocet transponovanej matice
   VSTUP:  -
   VYSTUP: -
*/
{
	double tmp = m_aadItem [0][1];
	m_aadItem [0][1] = m_aadItem [1][0];
	m_aadItem [1][0] = tmp;

	tmp = m_aadItem [0][2];
	m_aadItem [0][2] = m_aadItem [2][0];
	m_aadItem [2][0] = tmp;

	tmp = m_aadItem [1][2];
	m_aadItem [1][2] = m_aadItem [2][1];
	m_aadItem [2][1] = tmp;
}


const double CMatrix3::Determinant () const
/* UCEL:   vypocet determinantu matice
   VSTUP:  -
   VYSTUP: determinant matice
*/
{
	return (m_aadItem [0][0] * m_aadItem [1][1] * m_aadItem [2][2] +
			m_aadItem [1][0] * m_aadItem [2][1] * m_aadItem [0][2] +
			m_aadItem [2][0] * m_aadItem [0][1] * m_aadItem [1][2] -
			m_aadItem [0][2] * m_aadItem [1][1] * m_aadItem [2][0] -
			m_aadItem [1][2] * m_aadItem [2][1] * m_aadItem [0][0] -
			m_aadItem [2][2] * m_aadItem [0][1] * m_aadItem [1][0]);
}


const CMatrix3 CMatrix3::operator+ (const CMatrix3& rkM) const
/* UCEL:   sucet dvoch matic
   VSTUP:  rkM - pravy operand suctu
   VYSTUP: sucet matice reprezentovanej instanciou a parametra
*/
{
	CMatrix3 kResult;
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			kResult [i][j] = m_aadItem[i][j] + rkM[i][j];
	return kResult;
}


const CMatrix3 CMatrix3::operator- (const CMatrix3& rkM) const
/* UCEL:   rozdiel dvoch matic
   VSTUP:  rkM - pravy operand rozdielu
   VYSTUP: rozdiel matice reprezentovanej instanciou a parametra
*/
{
	CMatrix3 kResult;
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			kResult [i][j] = m_aadItem[i][j] - rkM[i][j];
	return kResult;
}


const CMatrix3 CMatrix3::operator* (const CMatrix3& rkM) const
/* UCEL:   sucin dvoch matic
   VSTUP:  rkM - pravy operand sucinu
   VYSTUP: sucin matice reprezentovanej instanciou a parametra
*/
{
	CMatrix3 kResult;
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
		{
			kResult [i][j] = 0;
			for (int k=0; k<3; k++)
				kResult [i][j] += m_aadItem[i][k] * rkM[k][j];
		}
	return kResult;
}


const CMatrix3 CMatrix3::operator* (const double dFactor) const
/* UCEL:   sucin matice a skalara
   VSTUP:  dFactor - skalarny operand suctu
   VYSTUP: sucin matice reprezentovanej instanciou a parametra
*/
{
	CMatrix3 kResult;
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			kResult [i][j] = m_aadItem[i][j] * dFactor;
	return kResult;
}


const CMatrix3 CMatrix3::operator+ (const double dFactor) const
/* UCEL:   sucet matice a skalara
   VSTUP:  dFactor - pravy operand suctu
   VYSTUP: sucet matice reprezentovanej instanciou a parametra
*/
{
	CMatrix3 kResult;
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			kResult [i][j] = m_aadItem[i][j] + dFactor;
	return kResult;
}


const CVector3 CMatrix3::operator* (const CVector3& rkVector) const
/* UCEL:   sucin matice a vektora
   VSTUP:  rkVector - pravy operand sucinu
   VYSTUP: sucin matice reprezentovanej instanciou a parametra
*/
{
	CVector3 kResult;
	for (int i=0; i<3; i++)
	{
		kResult [i] = 0;
		for (int j=0; j<3; j++)
			kResult [i] += m_aadItem[i][j] * rkVector [j];
	}
	return kResult;
}


CMatrix3& CMatrix3::operator= (const CMatrix3& rkM) 
/* UCEL:   priradenie matice
   VSTUP:  rkM - pravy operand priradenia
   VYSTUP: referencia na objekt vzniknuty priradenim
*/
{
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			m_aadItem[i][j] = rkM [i][j];
	return (*this);
}


const bool CMatrix3::operator== (const CMatrix3& rkM) const
/* UCEL:   zistenie rovnosti matic
   VSTUP:  rkM - pravy operand porovnavania
   VYSTUP: true ak su matice rovnake inak false
*/
{
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			if ( m_aadItem[i][j] != rkM [i][j] )
				return false;
	return true;
}


const bool CMatrix3::operator!= (const CMatrix3& rkM) const
/* UCEL:   zistenie nerovnosti matic
   VSTUP:  rkM - pravy operand porovnavania
   VYSTUP: true ak su matice rozne inak false
*/
{
	for (int i=0; i<3; i++)
		for (int j=0; j<3; j++)
			if ( m_aadItem[i][j] == rkM [i][j] )
				return false;
	return true;
}


void CMatrix3::FromAngleAxis (const CVector3 kAxis, const double dAngleDeg)
/* UCEL:   vytvorenie rotacnej matice
   VSTUP:  kAxis - os otacania
	   dAngleDeg - uhol otacania v stupnoch
   VYSTUP: -
*/
{

	//standardny tvar rotacnej matice
	//podrobna teoria sa da najst kdekolvek na internete
	//napr. http://gamedev.net
  
	double dAngleRad = 3.1415926538/180.0 * dAngleDeg;
	double dCos = cos (dAngleRad);
  double dSin = sin (dAngleRad);
  double dOneMinusCos = 1.0f-dCos;
  double dX2 = kAxis.X () * kAxis.X ();
  double dY2 = kAxis.Y () * kAxis.Y ();
  double dZ2 = kAxis.Z () * kAxis.Z ();
  double dXYM = kAxis.X () * kAxis.Y () * dOneMinusCos;
  double dXZM = kAxis.X () * kAxis.Z () * dOneMinusCos;
  double dYZM = kAxis.Y () * kAxis.Z () * dOneMinusCos;
  double dXSin = kAxis.X () * dSin;
  double dYSin = kAxis.Y () * dSin;
  double dZSin = kAxis.Z () * dSin;
    
  m_aadItem[0][0] = dX2 * dOneMinusCos + dCos;
  m_aadItem[0][1] = dXYM - dZSin;
  m_aadItem[0][2] = dXZM + dYSin;
  m_aadItem[1][0] = dXYM + dZSin;
  m_aadItem[1][1] = dY2 * dOneMinusCos + dCos;
  m_aadItem[1][2] = dYZM - dXSin;
  m_aadItem[2][0] = dXZM - dYSin;
  m_aadItem[2][1] = dYZM + dXSin;
  m_aadItem[2][2] = dZ2 * dOneMinusCos + dCos;
}