#include "stdafx.h"
#include "mesh.h"
#include "BlueBoneDlg.h"

CMesh::CMesh(void) : m_FaceCount(0), m_VertexCount(0), 
m_Vertices(NULL), m_Faces(NULL), m_fMaterialPower(0)
{
	ZeroMemory(&m_afMaterialAmbient, 4*sizeof(GLfloat));
	ZeroMemory(&m_afMaterialDiffuse, 4*sizeof(GLfloat));
	ZeroMemory(&m_afMaterialSpecular, 4*sizeof(GLfloat));
	ZeroMemory(&m_afMaterialEmissive, 4*sizeof(GLfloat));
    m_offset.x = 0;
    m_offset.y = 0;
    m_offset.z = 0;
}

CMesh::CMesh(const CMesh& rkOperand)
{

	m_VertexCount = rkOperand.m_VertexCount;
	m_NormalCount = rkOperand.m_NormalCount;
	m_FaceCount = rkOperand.m_FaceCount;

	m_Vertices = new SVertex[m_VertexCount];
	memcpy(m_Vertices, rkOperand.m_Vertices, m_VertexCount*sizeof(SVertex));

	m_Normals = new SNormal[m_NormalCount];
	memcpy(m_Normals, rkOperand.m_Normals, m_NormalCount*sizeof(SNormal));

	m_Faces = new SFace[m_FaceCount];
	memcpy(m_Faces, rkOperand.m_Faces, m_FaceCount*sizeof(SFace));

    m_offset.x = rkOperand.m_offset.x;
    m_offset.y = rkOperand.m_offset.y;
    m_offset.z = rkOperand.m_offset.z;
}

CMesh::~CMesh(void)
{
	if (m_Vertices)
		delete [] m_Vertices;
	if (m_Normals)
		delete [] m_Normals;
	if (m_Faces)
		delete [] m_Faces;
}

void CMesh::Render() const
{
  glBegin(GL_TRIANGLES);
	for (unsigned int i=0; i<m_FaceCount; i++)
	{
		glNormal3fv(&m_Normals[m_Faces[i].indexN1].x);
		glVertex3fv(&m_Vertices[m_Faces[i].indexV1].x);

		glNormal3fv(&m_Normals[m_Faces[i].indexN2].x);
		glVertex3fv(&m_Vertices[m_Faces[i].indexV2].x);

		glNormal3fv(&m_Normals[m_Faces[i].indexN3].x);
		glVertex3fv(&m_Vertices[m_Faces[i].indexV3].x);
	}
	glEnd();
	CBlueBoneDlg *dlg;
	dlg = (CBlueBoneDlg *)theApp.m_pMainWnd;
	if(dlg->m_showAxes) DrawAxes();
}

void CMesh::LoadFromFile(const char* acFileName)
{ 
  FILE *file;
  char buf[128];
  int numVertices = 0, numNormals = 0, numFaces = 0;
  SVertex vertex;
  SNormal normal;
  SFace face;
  std::vector<SVertex> vertices;
  std::vector<SNormal> normals;
  std::vector<SFace> faces;

  file = fopen(acFileName, "r");
  if (!file) {
    fprintf(stderr, "can't open data file \"%s\"..\n", acFileName);
    exit(1);
  }
	  
  ZeroMemory(&vertex, sizeof(SVertex));
  ZeroMemory(&normal, sizeof(SNormal));
  ZeroMemory(&face, sizeof(SFace));

  while (fscanf(file, "%s", buf) != EOF) {
    switch(buf[0]) {
      case '#':				// v poznamke musi byt udaj so suradnicami gule
	    if (buf[1] == 'x') {
		  fscanf(file, "%f %f %f", &m_offset.x, &m_offset.y, &m_offset.z);
		}
		/* eat up rest of line */
		fgets(buf, sizeof(buf), file);
        break;
      case 'v':				/* v, vn */
        switch(buf[1]) {
          case '\0':			/* vertex */
	        fscanf(file, "%f %f %f", &vertex.x, &vertex.y, &vertex.z);
            //odpocitame suradnice gule,
            //aby bol stred tam, kde sa bude objekt rotovat
            vertex.x -= m_offset.x;
            vertex.y -= m_offset.y;
            vertex.z -= m_offset.z;
			vertices.push_back(vertex);
			ZeroMemory(&vertex, sizeof(SVertex));
		    break;
	      case 'n':				/* normal */
	        fscanf(file, "%f %f %f", &normal.x, &normal.y, &normal.z);
            //invertujeme normaly, lebo asi Wavefront OBJ pouziva opacne
            normal.x = -normal.x;
            normal.y = -normal.y;
            normal.z = -normal.z;
			normals.push_back(normal);
			ZeroMemory(&normal, sizeof(SNormal));
	        break;
		}
        break;
      case 'f':				/* face */
        fscanf(file, "%s", buf);
        /* v//n */
        if (strstr(buf, "//")) {
		  sscanf(buf, "%d//%d", &face.indexV1, &face.indexN1); 
	      fscanf(file, "%d//%d", &face.indexV2, &face.indexN2);
	      fscanf(file, "%d//%d", &face.indexV3, &face.indexN3);
		  // dekrementujeme, lebo pole zacina indexom 0
		  face.indexV1--;
		  face.indexV2--;
		  face.indexV3--;
		  face.indexN1--;
		  face.indexN2--;
		  face.indexN3--;
		}
		faces.push_back(face);
		ZeroMemory(&face, sizeof(SFace));
        break;
	  default:
      /* eat up rest of line */
        fgets(buf, sizeof(buf), file);
        break;
    }
  }

  m_VertexCount = vertices.size();
  m_NormalCount = normals.size();
  m_FaceCount = faces.size();
  

  m_Vertices = new SVertex[m_VertexCount];
  for (int i=0; i<m_VertexCount; i++)
    m_Vertices[i] = vertices[i];

  m_Normals = new SNormal[m_NormalCount];
  for (i=0; i<m_NormalCount; i++)
    m_Normals[i] = normals[i];

  m_Faces = new SFace[m_FaceCount];
  for (i=0; i<m_FaceCount; i++)
	m_Faces[i] = faces[i];	

//  Unitize();

  GLfloat scale = (GLfloat) 0.06698;
  for (i=0; i<m_VertexCount; i++) {
    m_Vertices[i].x *= scale;
    m_Vertices[i].y *= scale;
    m_Vertices[i].z *= scale;
  }
  m_offset.x *= scale;
  m_offset.y *= scale;
  m_offset.z *= scale;

//  IssueMaterial();
}

void CMesh::IssueMaterial(void)
{
	glMaterialfv(GL_FRONT, GL_AMBIENT, m_afMaterialAmbient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, m_afMaterialDiffuse);
	glMaterialfv(GL_FRONT, GL_EMISSION, m_afMaterialEmissive);
	glMaterialfv(GL_FRONT, GL_SPECULAR, m_afMaterialSpecular);
	glMaterialf(GL_FRONT, GL_SHININESS, m_fMaterialPower);
}

SVertex * CMesh::GetOffsetVertex()
{
    return &m_offset;
}

void CMesh::DrawAxes() const
{
	GLfloat color[4];
	color[0]=color[2]=0.0f;
	color[1]=color[3]=1.0f;
	GLfloat scale = 8.0f;
	glColor3fv(color);
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
		for(int i = 0; i<m_kAxes.size(); i++){
			GLfloat axe[3];
			axe[0] = m_kAxes[i].x*scale;
			axe[1] = m_kAxes[i].y*scale;
			axe[2] = m_kAxes[i].z*scale;
			glVertex3f(-axe[0], -axe[1], -axe[2]);
			glVertex3f(axe[0], axe[1], axe[2]);
		}
    glEnd();
	glEnable(GL_LIGHTING);
}
