#include "stdafx.h"
#include "node.h"

CNode::CNode(CMesh* pkMesh /* = NULL */)
: m_uiID(0)
, m_bHighlight(false)
{
	glPushMatrix();
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX, m_afTranslationMatrix);
	glGetFloatv(GL_MODELVIEW_MATRIX, m_afRotationMatrix);
	glPopMatrix();

	if (pkMesh)
		SetMesh(pkMesh);
	else
		pkMesh = NULL;

	m_afHighlightColor[0] = m_afHighlightColor[1] = m_afHighlightColor[2] = 
		m_afHighlightColor[3] = 1.0f;
}

CNode::~CNode()
{
	for (unsigned int i=0; i<m_kChildren.size(); i++)
		delete m_kChildren[i];
	if (m_pkMesh)
		delete m_pkMesh;
}

void CNode::Render() const
{
	glMultMatrixf(m_afTranslationMatrix);
	glMultMatrixf(m_afRotationMatrix);

	if (m_bHighlight)
	{
		glPushAttrib(GL_LIGHTING_BIT);
		glColor3fv(m_afHighlightColor);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
		glEnable(GL_COLOR_MATERIAL);
	}
	
	DrawMesh();

	if (m_bHighlight)
	{
		glDisable(GL_COLOR_MATERIAL);
		glPopAttrib();
	}

	for (unsigned int i=0; i<m_kChildren.size(); i++)
	{
		glPushMatrix();
		m_kChildren[i]->Render();
		glPopMatrix();
	}
}

void CNode::RotateBy(GLfloat fDegAngle, GLfloat* afAxis)
{
	glPushMatrix();
	glLoadMatrixf(m_afRotationMatrix);
	glRotatef(fDegAngle, afAxis[0], afAxis[1], afAxis[2]);
	glGetFloatv(GL_MODELVIEW_MATRIX, m_afRotationMatrix);
	glPopMatrix();
}

void CNode::MoveBy(SVertex* offset, bool substract)
{
	glPushMatrix();
	glLoadMatrixf(m_afTranslationMatrix);
    if (substract)
        glTranslatef(-offset->x, -offset->y, -offset->z);
    else
        glTranslatef(offset->x, offset->y, offset->z);
	glGetFloatv(GL_MODELVIEW_MATRIX, m_afTranslationMatrix);
	glPopMatrix();
}

CNode* CNode::FindNode(const unsigned int uiID)
{
	if (m_uiID == uiID)
		return this;
	else if (m_kChildren.size() == 0)
		return NULL;
	else
	{
		for (unsigned int i=0; i<m_kChildren.size(); i++)
		{
			CNode* pkTarget = m_kChildren[i]->FindNode(uiID);
			if (pkTarget)
				return pkTarget;
		}
		return NULL;
	}
}

void CNode::ResetRotation(void)
{
	glPushMatrix();
	glLoadIdentity();
	glGetFloatv(GL_MODELVIEW_MATRIX, m_afRotationMatrix);
	glPopMatrix();
	for (unsigned int i=0; i<m_kCurrentAngles.size(); i++)
		m_kCurrentAngles[i] = 0.0;
}
