#pragma once
#include "Mesh.h"

#include <vector>

class CNode
{
public:
	CNode(CMesh* pkMesh = NULL);
	~CNode();
private:
	GLfloat m_afTranslationMatrix[16];
	GLfloat m_afRotationMatrix[16];

	CMesh* m_pkMesh;
	std::vector<CNode*> m_kChildren;
	std::vector<SAxis> m_kAxes;
public:
	void Render() const;

	void RotateBy(GLfloat fDegAngle, GLfloat* afAxis);
    // posunutie (ked substract = true, tak sa odpocitava)
	void MoveBy(SVertex* offset, bool substract = false);

	const GLfloat* GetTranslationMatrix() const
	{
		return m_afTranslationMatrix;
	}

	const GLfloat* GetRotationMatrix() const
	{
		return m_afRotationMatrix;
	}

	void DrawMesh() const
	{
		glLoadName(m_uiID);
		m_pkMesh->Render();
	}

	void SetMesh(CMesh* pkMesh)
	{
		m_pkMesh = pkMesh;
	}

	const CMesh* GetMesh() const
	{
		return m_pkMesh;
	}

	void AddChildNode(CNode* pkChild)
	{
		m_kChildren.push_back(pkChild);
	}
private:
	unsigned int m_uiID;
public:

	const unsigned int GetID(void)
	{
		return m_uiID;
	}

	void SetID(const unsigned int uiID)
	{
		m_uiID = uiID;
	}
	CNode* FindNode(const unsigned int uiID);
private:
	bool m_bHighlight;
	GLfloat m_afHighlightColor[4];
public:

	void SetHighlightColor(const GLfloat* af_Color)
	{
		memcpy(m_afHighlightColor, af_Color, 4*sizeof(GLfloat));
	}

	const GLfloat* GetHighlightColor() const
	{
		return m_afHighlightColor;
	}

	void EnableHighlight(bool bEnable = true)
	{
		m_bHighlight = bEnable;
	}

	void AddAxis(const GLfloat* afVector)
	{
    SAxis axis;
		ZeroMemory(&axis, sizeof(SAxis));
		axis.x = afVector[0];
		axis.y = afVector[1];
		axis.z = afVector[2];
		axis.l_limit = afVector[3];	//lower restriction limit
		axis.u_limit = afVector[4];	//upper restriction limit
		m_kAxes.push_back(axis);
		m_kCurrentAngles.push_back(0);
		m_kSnapshotAngles.push_back(0);
		m_pkMesh->AddAxis(afVector);
	}

	void RotateAroundAxis(GLfloat fDegAngle, unsigned int uiIndex)
	{
    GLfloat fAuxAngle = 0;
		if (uiIndex < m_kAxes.size())
		{
      // vypocitame novy aktualny uhol rotacie
			GLfloat fAngle = m_kCurrentAngles[uiIndex] + fDegAngle;
      // ak je tento uhol medzi hornym a dolnym limitom obmedzujucim rotaciu v danom klbe
      // alebo ak sa v danom klbe moze kost ohybat bez obmedzenia, tak uskutocnime rotaciu
			if ( (fAngle > m_kAxes[uiIndex].l_limit && fAngle < m_kAxes[uiIndex].u_limit) ||
				(m_kAxes[uiIndex].l_limit == 0.0 && m_kAxes[uiIndex].u_limit == 0.0) )
			{
				this->RotateBy(fDegAngle, (GLfloat*)&m_kAxes[uiIndex]);
				m_kCurrentAngles[uiIndex] = fAngle;
			}
      // ak by sme mali prekrocit spodnu hranicu obmedzenia ohybania, tak v danom klbe 
      // ohneme kostru len po tento limit
      else if ( fAngle < m_kAxes[uiIndex].l_limit )
      {
        fAuxAngle = m_kAxes[uiIndex].l_limit - m_kCurrentAngles[uiIndex];
        this->RotateBy(fAuxAngle, (GLfloat*)&m_kAxes[uiIndex]);
        m_kCurrentAngles[uiIndex] = m_kAxes[uiIndex].l_limit;
      }
      // ak by sme mali prekrocit hornu hranicu obmedzenia ohybania, tak v danom klbe 
      // ohneme kostru len po tento limit
      else if ( fAngle > m_kAxes[uiIndex].u_limit )
      {
        fAuxAngle = m_kAxes[uiIndex].u_limit - m_kCurrentAngles[uiIndex];
        this->RotateBy(fAuxAngle, (GLfloat*)&m_kAxes[uiIndex]);
        m_kCurrentAngles[uiIndex] = m_kAxes[uiIndex].u_limit;
      }
		}
	}
private:
	std::vector<GLfloat> m_kCurrentAngles, m_kSnapshotAngles;
public:

	void RemoveAxes(void)
	{
		m_kCurrentAngles.clear();
		m_kSnapshotAngles.clear();
		m_kAxes.clear();
	}
	void ResetRotation(void);
	void MakeSnapshot()
	{
		m_kSnapshotAngles.clear();
		for (unsigned int i=0; i<m_kCurrentAngles.size(); i++)
			m_kSnapshotAngles.push_back(m_kCurrentAngles[i]);
	}

	void GetSnapshot(std::vector<GLfloat>& rkSnapshot) const
	{
		for (unsigned int i=0; i<m_kSnapshotAngles.size(); i++)
			rkSnapshot.push_back(m_kSnapshotAngles[i]);
	}

	const std::vector<SAxis> GetAxes() const
	{
		return m_kAxes;
	}
};
