#pragma once
#include <assert.h>
#include <iostream>
#include "math.h"
#include "mesh.h"

#define pi 3.14159265358979323846

class CVector3
{
public:

	/* UCEL:   kopirovaci konstruktor
	   VSTUP:  rkVec - referencny objekt na kopirovanie
	   VYSTUP: -
	*/
	CVector3 (const CVector3& rkVec) 
		: m_dX(rkVec.m_dX),	m_dY(rkVec.m_dY), m_dZ(rkVec.m_dZ)
  { }


	/* UCEL:   konstruktor 1
	   VSTUP:  dX - 1. suradnica vektora
		   dY - 2. suradnica vektora
		   dZ - 3. suradnica vektora
	   VYSTUP: -
	*/
	CVector3 (const double dX = 0.0, const double dY = 0.0, const double dZ = 0.0)
		: m_dX(dX), m_dY(dY), m_dZ(dZ)
  { }


	/* UCEL:   konstruktor 2
	   VSTUP:  adArray - pole suradnic vektora
	   VYSTUP: -
	*/
	CVector3 (const double adArray [3])
		: m_dX(adArray[0]), m_dY(adArray[1]), m_dZ(adArray[2])
  { }


	/* UCEL:   konstruktor 3
	   VSTUP:  v1 - zaciatocny vrchol vektora, v2  - koncovy vrchol vektora
	   VYSTUP: -
	*/
  CVector3 (SVertex v1, SVertex v2)
		: m_dX(v2.x - v1.x), m_dY(v2.y - v1.y), m_dZ(v2.z - v1.z)
  { }


	/* UCEL:   destruktor
	   VSTUP:  -
	   VYSTUP: -
	*/
	~CVector3 () 
  { }


	/* UCEL:   funkcia zostavi vektor z dvoch vrcholov (presne to iste ako konstruktor 3)
	   VSTUP:  v1 - zaciatocny vrchol vektora, v2  - koncovy vrchol vektora
	   VYSTUP: -
	*/
  void FromVertices(SVertex v1, SVertex v2)
  {
    m_dX = v2.x - v1.x;
    m_dY = v2.y - v1.y;
    m_dZ = v2.z - v1.z;
  }


	/* UCEL:   sucet 2 vektorov
	   VSTUP:  rkVec - pravy operand operacie
	   VYSTUP: sucet vektora reprezentovaneho instanciou a parametra
	*/
	CVector3 operator+ (const CVector3& rkVec) 
	{
		return CVector3 (m_dX + rkVec.m_dX, m_dY + rkVec.m_dY, m_dZ + rkVec.m_dZ);
	}


	/* UCEL:   rozdiel 2 vektorov
	   VSTUP:  rkVec - pravy operand operacie
	   VYSTUP: rozdiel vektora reprezentovaneho instanciou a parametra
	*/
	CVector3 operator- (const CVector3& rkVec)
	{
		return CVector3 (m_dX - rkVec.m_dX, m_dY - rkVec.m_dY, m_dZ - rkVec.m_dZ);
	}


	/* UCEL:   sucin vektora a skalara
	   VSTUP:  dFactor - skalarny operand operacie
	   VYSTUP: sucin vektora reprezentovaneho instanciou a parametra
	*/
	CVector3 operator* (const double dFactor) const
	{
		return CVector3 (m_dX * dFactor, m_dY * dFactor, m_dZ * dFactor);
	}


	/* UCEL:   podiel vektora a skalara
	   VSTUP:  dFactor - skalarny operand operacie
	   VYSTUP: podiel vektora reprezentovaneho instanciou a parametra
	*/
	CVector3 operator/ (const double dFactor)
	{
		assert (dFactor != 0);
		return CVector3 (m_dX / dFactor, m_dY / dFactor, m_dZ / dFactor);
	}


	/* UCEL:   priradovaci operator
	   VSTUP:  rkVec - referencny objekt na priradenie
	   VYSTUP: referencia na objekt vzniknuty priradenim
	*/
	CVector3& operator= (const CVector3& rkVec)
	{
		m_dX = rkVec.m_dX;
		m_dY = rkVec.m_dY;
		m_dZ = rkVec.m_dZ;
		return *this;
	}


	/* UCEL:   operator rovnosti
	   VSTUP:  rkVec - pravy operand operacie
	   VYSTUP: true ak sa vektory rovnaju inak false
	*/
	bool operator== (const CVector3& rkVec)
	{
		return ((m_dX == rkVec.m_dX) && (m_dY == rkVec.m_dY) && (m_dZ == rkVec.m_dZ));
	}


	/* UCEL:   sucet dvoch vektorov
	   VSTUP:  rkVec - pravy operand operacie
	   VYSTUP: sucet vektora reprezentovaneho instanciou a parametra
	*/
	CVector3& operator+= (const CVector3& rkVec)
	{
		m_dX += rkVec.m_dX;
		m_dY += rkVec.m_dY;
		m_dZ += rkVec.m_dZ;
		return *this;
	}
	

	/* UCEL:   rozdiel 2 vektorov
	   VSTUP:  rkVec - pravy operand operacie
	   VYSTUP: rozdiel vektora reprezentovaneho instanciou a parametra
	*/
	CVector3& operator-= (const CVector3& rkVec)
	{
		m_dX -= rkVec.m_dX;
		m_dY -= rkVec.m_dY;
		m_dZ -= rkVec.m_dZ;
		return *this;
	}


	/* UCEL:   sucin vektora a skalara
	   VSTUP:  dFactor - skalarny operand operacie
	   VYSTUP: sucin vektora reprezentovaneho instanciou a parametra
	*/
	CVector3& operator*= (const double dFactor)
	{
		m_dX *= dFactor;
		m_dY *= dFactor;
		m_dZ *= dFactor;
		return *this;
	}


	/* UCEL:   podiel vektora a skalara
	   VSTUP:  dFactor - skalarny operand operacie
	   VYSTUP: podiel vektora reprezentovaneho instanciou a parametra
	*/
	CVector3& operator/= (const double dFactor)
	{
		assert (dFactor != 0);
		m_dX /= dFactor;
		m_dY /= dFactor;
		m_dZ /= dFactor;
		return *this;
	}


	/* UCEL:   indexovaci operator
	   VSTUP:  iX - index suradnice
	   VYSTUP: referencia na suradnicu
	*/
	double& operator[] (int iX)
	{
		if (iX == 0)
			return m_dX;
		else if (iX == 1)
			return m_dY;
		else return m_dZ;
	}


	/* UCEL:   indexovaci operator
	   VSTUP:  iX - index suradnice
	   VYSTUP: hodnota suradnice
	*/
	const double operator[] (int iX) const
	{
		if (iX == 0)
			return m_dX;
		else if (iX == 1)
			return m_dY;
		else return m_dZ;
	}


	/* UCEL:   vystup vektora
	   VSTUP:  rkStream - vystupny prud
		   rkVec - vektor na vystup
	   VYSTUP: novy vystupny prud
	*/
	friend std::ostream& operator<< (std::ostream& rkStream, const CVector3& rkVec)
	{
		rkStream << "(" << rkVec.m_dX << "," << rkVec.m_dY << "," << rkVec.m_dZ << ")" << std::endl;
		return rkStream;
	}


	/* UCEL:   spristupnenie 1. suradnice vektora
	   VSTUP:  -
	   VYSTUP: referencia na 1. suradnicu vektora
	*/
	double& X () 
	{return m_dX;}


	/* UCEL:   spristupnenie 1. suradnice vektora
	   VSTUP:  -
	   VYSTUP: hodnota 1. suradnice vektora
	*/
	const double X () const 
	{return m_dX;}


	/* UCEL:   spristupnenie 2. suradnice vektora
	   VSTUP:  -
	   VYSTUP: referencia na 2. suradnicu vektora
	*/
	double& Y () 
	{return m_dY;}


	/* UCEL:   spristupnenie 2. suradnice vektora
	   VSTUP:  -
	   VYSTUP: hodnota 2. suradnice vektora
	*/
	const double Y () const 
	{return m_dY;}


	/* UCEL:   spristupnenie 3. suradnice vektora
	   VSTUP:  -
	   VYSTUP: referencia na 3. suradnicu vektora
	*/
	double& Z () 
	{return m_dZ;}


	/* UCEL:   spristupnenie 3. suradnice vektora
	   VSTUP:  -
	   VYSTUP: hodnota 3. suradnice vektora
	*/
	const double Z () const 
	{return m_dZ;}

	/* UCEL:   zistenie dlzky vektora
	   VSTUP:  -
	   VYSTUP: hodnota dlzky vektora
	*/
	const double Length () const 
	{ return sqrt (m_dX*m_dX + m_dY*m_dY + m_dZ*m_dZ); }


	/* UCEL:   zistenie stvorca dlzky vektora
	   VSTUP:  -
	   VYSTUP: hodnota stvorca dlzky vektora
	*/
	const double SquaredLength () const 
	{ return (m_dX*m_dX + m_dY*m_dY + m_dZ*m_dZ); }
	

	/* UCEL:   vypocet skalarneho sucinu dvoch vektorov
	   VSTUP:  rkVector - pravy operand sucinu
	   VYSTUP: skalarny sucin vektora reprezentovaneho instanciou a parametra
  */
  double Scalar(const CVector3& rkVector) const
	{
		return m_dX * rkVector.m_dX + m_dY * rkVector.m_dY + m_dZ * rkVector.m_dZ;
	}

	/* UCEL:   vypocet uhlu medzi dvomi vektormi
	   VSTUP:  rkVector - druhy vektor
	   VYSTUP: uhol medzi vektorom reprezentovanym instanciou a vektorom reprezentovanym parametrom
  */
  double Angle(const CVector3& rkVector) const
	{
    double angle = acos( Scalar(rkVector) / (Length()*rkVector.Length()) );
    angle *= 180/pi; //prevod z radianov na stupne
    return angle;
    //return angle<0 ? -1*angle : angle;
	}

  double Angle2(const CVector3& rkVector) const
	{
    double angle = asin( Cross(rkVector).Length() / (Length()*rkVector.Length()) );
    angle *= 180/pi; //prevod z radianov na stupne
    return angle;
    //return angle<0 ? -1*angle : angle;
	}


	/* UCEL:   vypocet vektoroveho sucinu
	   VSTUP:  rkVector - pravy operand sucinu
	   VYSTUP: vektorovy sucin vektora reprezentovaneho instanciou a parametra
    */
	CVector3 Cross (const CVector3& rkVector) const
	{
		return CVector3(m_dY*rkVector.Z () - m_dZ*rkVector.Y (),
			m_dZ*rkVector.X () - m_dX*rkVector.Z (),
			m_dX*rkVector.Y () - m_dY*rkVector.X ());
	}
	
	/* UCEL:   normalizacia vektora
	   VSTUP:  -
	   VYSTUP: -
    */
	void Normalize ()
	{
		double dLength = Length ();
		m_dX /= dLength;
		m_dY /= dLength;
		m_dZ /= dLength;
	}
  
protected:
	double m_dX, m_dY, m_dZ;

public:
	static const CVector3 ZERO;
	static const CVector3 UNIT;
	static const CVector3 UNIT_X;
	static const CVector3 UNIT_Y;
	static const CVector3 UNIT_Z;
};

