// glWnd.cpp : implementation file
//

#include "stdafx.h"
#include "BlueBone.h"
#include "glWnd.h"
#include "BlueBoneDlg.h"

// CglWnd

IMPLEMENT_DYNAMIC(CglWnd, CWnd)
CglWnd::CglWnd(CWnd* pkWindow) : m_pkWnd(pkWindow), m_iLastMouseX(-1), m_iLastMouseY(-1),
	m_pkLastActiveNode(NULL)
{
	m_hWnd = m_pkWnd->m_hWnd;
	m_hDC = ::GetDC(m_hWnd);
	this->OnCreate();
	ResetCamera();
}

CglWnd::~CglWnd()
{
	if (m_pkCamera)
		delete m_pkCamera;
}


BEGIN_MESSAGE_MAP(CglWnd, CWnd)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
END_MESSAGE_MAP()



// CglWnd message handlers


bool CglWnd::SetPixelFormat()
{
	PIXELFORMATDESCRIPTOR *ppfd; 
	int pixelformat; 

	PIXELFORMATDESCRIPTOR pfd = { 
		sizeof(PIXELFORMATDESCRIPTOR),  //  size of this pfd 
			1,                     // version number 
			PFD_DRAW_TO_WINDOW |   // support window 
			PFD_SUPPORT_OPENGL |   // support OpenGL 
			PFD_GENERIC_FORMAT |
			PFD_DOUBLEBUFFER,      // double buffered 
			PFD_TYPE_RGBA,         // RGBA type 
			16/*32*/,                    // 24-bit color depth 
			0, 0, 0, 0, 0, 0,      // color bits ignored 
			8,                     // no alpha buffer 
			0,                     // shift bit ignored 
			8,                     // no accumulation buffer 
			0, 0, 0, 0,            // accum bits ignored 
			16/*64*/,                    // 32-bit z-buffer	 
			8,                     // no stencil buffer 
			8,                     // no auxiliary buffer 
			PFD_MAIN_PLANE,        // main layer 
			0,                     // reserved 
			0, 0, 0                // layer masks ignored 
	}; 

	ppfd = &pfd;

	if ( (pixelformat = ChoosePixelFormat(m_hDC, ppfd)) == 0 ) 
	{ 
		::MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
		return false; 
	} 

	if (::SetPixelFormat(m_hDC, pixelformat, ppfd) == FALSE) 
	{ 
		::MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
		return false; 
	} 

	return true; 
}

void CglWnd::OnResize(int iWidth, int iHeight)
{
	if (iHeight == 0)
		iHeight = 1;

	glViewport(0, 0, iWidth, iHeight);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f, (GLfloat)iWidth/(GLfloat)iHeight, 0.1f, 1000.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void CglWnd::InitGL(void)
{
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	//glPolygonMode(GL_FRONT, GL_LINE);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

	GLfloat afLightAmbientDiffuse[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat afLightPosition[] = {0.0, 0.0, -1.0, 0.0};
	
	glLightfv(GL_LIGHT0, GL_AMBIENT_AND_DIFFUSE, afLightAmbientDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, afLightPosition);

	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
}

void CglWnd::DrawScene(CNode* pkRoot)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	m_pkCamera->SetViewMatrix();
	pkRoot->Render();

	SwapBuffers(m_hDC);
}

bool CglWnd::OnCreate()
{
	if(!SetPixelFormat())
	{
		::MessageBox(::GetFocus(),"SetPixelformat Failed!","Error",MB_OK);
		return false;
	}

	m_hglRC = wglCreateContext(m_hDC);
	wglMakeCurrent(m_hDC, m_hglRC);

	RECT rect;
	m_pkWnd->GetClientRect(&rect);
	OnResize(rect.right - rect.left, rect.bottom - rect.top);

	InitGL();	

	return true;
}

void CglWnd::OnMouseMove(UINT nFlags, CPoint point)
{
	unsigned int uiObject = 0;
	
  if (nFlags == 0) uiObject = DoPick(point.x, point.y);

  if (uiObject != 0)
	{
        GLfloat afHighlightColor[] = {0.17, 0.55, 0.97, 1.0};
		CNode* pkRootNode = ((CBlueBoneDlg*)AfxGetMainWnd())->GetRootNode();
		CNode* pkActiveNode = pkRootNode->FindNode(uiObject);
		if (pkActiveNode != m_pkLastActiveNode)
		{
      ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;

			pkActiveNode->SetHighlightColor(afHighlightColor);
			pkActiveNode->EnableHighlight(true);
			
			if (m_pkLastActiveNode)
			{
				afHighlightColor[0] = afHighlightColor[1] = afHighlightColor[2] = 1.0f;
				m_pkLastActiveNode->EnableHighlight(false);
				m_pkLastActiveNode->SetHighlightColor(afHighlightColor);
			}
			m_pkLastActiveNode = pkActiveNode;
		}
	}
	else if (m_pkLastActiveNode && nFlags == 0)
	{
		m_pkLastActiveNode->EnableHighlight(false);
		m_pkLastActiveNode = NULL;
    ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
	}

	CNode* pkCurrentNode = ((CBlueBoneDlg*)AfxGetMainWnd())->GetCurrentNode();
  // ak sme myskou neuchopili kost, tak robime len pohyb kamery
	if (!pkCurrentNode)
	{
		if (nFlags & MK_LBUTTON)
		{
			if (m_iLastMouseX != -1)
			{
				m_pkCamera->ArcX((point.x - m_iLastMouseX) * (-0.6));
				m_pkCamera->ArcY((point.y - m_iLastMouseY) * (0.6));
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
			}
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else if (nFlags & MK_RBUTTON)
		{
			if (m_iLastMouseX != -1)
			{
				m_pkCamera->StrafeRight((point.x - m_iLastMouseX) * (0.3));
				m_pkCamera->StrafeUp((point.y - m_iLastMouseY) * (0.3));
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
			}
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else
		{
			m_iLastMouseX = -1;
			m_iLastMouseY = -1;
		}
	}
  // ale ak drzime nejaku kost, tak potom uskutocnujeme rotaciu podla zvolenych osi
  // pripadne translaciu (ak drzime panvu a je stlacene CTRL)
	else
	{
    if (nFlags & MK_LBUTTON && !(nFlags & MK_RBUTTON) && (nFlags & MK_CONTROL) &&
      pkCurrentNode == ((CBlueBoneDlg*)AfxGetMainWnd())->GetRootNode())
		{
			if (m_iLastMouseX != -1)
      {
        SVertex v;
        v.x = (m_iLastMouseX - point.x) / 10.0;
        v.y = (m_iLastMouseY - point.y) / 10.0;
        v.z = 0;
				pkCurrentNode->MoveBy(&v);
        // a teraz pokrcenie noh  !!! treba dorobit !!!
        // -------------------------------------
        CNode* rootNode = ((CBlueBoneDlg*)AfxGetMainWnd())->GetRootNode();
        CNode* node = rootNode->FindNode(ID_L_FEMUR);
        
        // bedra
        node->RotateAroundAxis(v.y*3, 0);
        node = rootNode->FindNode(ID_R_FEMUR);
        node->RotateAroundAxis(v.y*3, 0);

        // kolena
        node = rootNode->FindNode(ID_L_LOWER_LEG);
        node->RotateAroundAxis(-v.y*5, 0);
        node = rootNode->FindNode(ID_R_LOWER_LEG);
        node->RotateAroundAxis(-v.y*5, 0);

        // clenky
        node = rootNode->FindNode(ID_L_FOOT);
        node->RotateAroundAxis(v.y*5, 0);
        node = rootNode->FindNode(ID_R_FOOT);
        node->RotateAroundAxis(v.y*5, 0);
        //---------------------------------------------
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
      }
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else if (nFlags & MK_LBUTTON && !(nFlags & MK_RBUTTON))
		{
			if (m_iLastMouseX != -1)
      {
				pkCurrentNode->RotateAroundAxis(-(m_iLastMouseY - point.y), 0);
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
      }
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else if (nFlags & MK_RBUTTON && !(nFlags & MK_LBUTTON))
		{
			if (m_iLastMouseX != -1)
      {
        pkCurrentNode->RotateAroundAxis((m_iLastMouseY - point.y), 1);
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
      }
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else if ((nFlags & MK_RBUTTON ) && (nFlags & MK_LBUTTON))
		{
			if (m_iLastMouseX != -1)
      {
				pkCurrentNode->RotateAroundAxis((m_iLastMouseY - point.y), 2);
        ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
      }
			m_iLastMouseX = point.x;
			m_iLastMouseY = point.y;
		}
		else
		{
			m_iLastMouseX = -1;
			m_iLastMouseY = -1;
		}
	}

	CWnd::OnMouseMove(nFlags, point);
}

BOOL CglWnd::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
    if (zDelta < 0)
    {
      m_pkCamera->MoveForward(-4);
    }
    else
    {
      m_pkCamera->MoveForward(4);
    }
    return true;
}

unsigned int CglWnd::DoPick(int iX, int iY)
{
	static int iSelectedObject = 0;
  unsigned int auSelectionBuffer [32] = {0};
	int aiViewport [4];
  GLubyte color[3];

  
  glSelectBuffer (32, auSelectionBuffer);
	glGetIntegerv (GL_VIEWPORT, aiViewport);

  // zistime si farbu pixelu pod kurzorom
  glReadPixels(iX, aiViewport[3] - iY, 1, 1, GL_RGB, GL_UNSIGNED_BYTE, (GLvoid*) &color);
	
  // ak to je cierna farba - teda ak nie sme nad kostrou, tak nic nerobime
  // ale ak to je ina farba ako cierna, tak musime zistit, nad ktorym objektom sa
  // kurzor nachadza
	if (color[0] + color[1] + color[2] != 0)
  {
    // kedze objekty vyznacujeme modrou farbou, tak vieme zistit, ci nie sme nad prave
    // vyznacenym objektom... ak ano, tak vratime jeho id, bez zbytocneho testovania
    if (color [0] + color [1] == 0) return iSelectedObject;

	  glMatrixMode (GL_PROJECTION);
	  glPushMatrix ();
	  glRenderMode (GL_SELECT);
	  glInitNames ();
	  glPushName (0);
	  glLoadIdentity ();
	  gluPickMatrix (iX, aiViewport[3] - iY, 5, 5, aiViewport);
	  gluPerspective (45.0, (GLfloat)(aiViewport [2] - aiViewport [0])/
		  (GLfloat)(aiViewport [3] - aiViewport [1]), 0.1, 1000.0);
	  glMatrixMode (GL_MODELVIEW);

    ((CBlueBoneDlg*)AfxGetMainWnd())->DrawRootNode();

	  int iHits = glRenderMode (GL_RENDER);
	  glMatrixMode (GL_PROJECTION);
	  glPopMatrix ();
	  glMatrixMode (GL_MODELVIEW);
	  if (iHits > 0)
	  {
		  unsigned int uLowestDepth = auSelectionBuffer [1];
		  iSelectedObject = auSelectionBuffer [3];

		  for (int i=1; i<iHits; i++)
		  {
			  if (auSelectionBuffer [(i * 4) + 1] < uLowestDepth)
			  {
				  uLowestDepth = auSelectionBuffer [(i * 4) + 1];
				  iSelectedObject = auSelectionBuffer [(i * 4) + 3];
			  }
		  }
		  return iSelectedObject;
	   }
  }

	return 0;
}

void CglWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
	unsigned int uiObject = DoPick(point.x, point.y);

	CWnd::OnLButtonDown(nFlags, point);
	((CBlueBoneDlg*)AfxGetMainWnd())->SetCurrentNode(uiObject);
}

void CglWnd::OnRButtonDown(UINT nFlags, CPoint point)
{
	unsigned int uiObject = DoPick(point.x, point.y);

	CWnd::OnRButtonDown(nFlags, point);
	((CBlueBoneDlg*)AfxGetMainWnd())->SetCurrentNode(uiObject);
}

void CglWnd::OnLButtonUp(UINT nFlags, CPoint point)
{
  unsigned int uiObject = DoPick(point.x, point.y);
  if (uiObject == 0 && m_pkLastActiveNode && nFlags == 0)
	{
		m_pkLastActiveNode->EnableHighlight(false);
		m_pkLastActiveNode = NULL;
    ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
	}
}

void CglWnd::OnRButtonUp(UINT nFlags, CPoint point)
{
  unsigned int uiObject = DoPick(point.x, point.y);
  if (uiObject == 0 && m_pkLastActiveNode && nFlags == 0)
	{
		m_pkLastActiveNode->EnableHighlight(false);
		m_pkLastActiveNode = NULL;
    ((CBlueBoneDlg*)AfxGetMainWnd())->redraw = true;
	}
}
void CglWnd::ResetCamera()
{
  m_pkCamera = new CCamera();
	m_pkCamera->Location() = CVector3(0.0, 0.0, 110.0);
	m_pkCamera->Up() = CVector3::UNIT_Y;
	m_pkCamera->Left() = CVector3(1.0, 0.0, 0.0);
	m_pkCamera->Direction() = CVector3(0.0, 0.0, -1.0);
}
