//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				AVIExtractor.cpp
// Ucel suboru:			definicia triedy CAVIExtractor
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Lucenic / 20.4.2005
// Poznamka:			Abstaraktna trieda, implementujuca pristup k AVI suborom,
//						za ucelom extrakcie obrazkov.
//						Autor: Juraj Hajek
//______________________________________________________________________________________________________


#include "stdafx.h"
#include "AVIExtractor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAXNUMSTREAMS 5 


CAviExtractor::CAviExtractor()
	: m_ppFile(NULL)
	, m_ppAvi(NULL)
{	
}


CAviExtractor::CAviExtractor(CString &strVideoFilename)
	: m_ppFile(NULL)
	, m_ppAvi(NULL)
{
	OpenFile(strVideoFilename);
}


CAviExtractor::~CAviExtractor()
{
    if(m_ppFile != NULL)
		CloseFile(); 	
}


// otvorenie a inicializacia AVI streamu
// param:	strVideoFilename - meno videosuboru
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::OpenFile(CString &strVideoFilename)
{
    PAVIFILE ppFile = NULL; 
    PAVISTREAM ppAvi = NULL;

	AVIFileInit();

	if (AVIFileOpen(&ppFile, strVideoFilename, OF_READ, 0L) != 0)
		return E_FAIL;
	
	// stream, s ktorym budeme pracovat, musi byt VIDEO stream...
    for(int i = 0; i < MAXNUMSTREAMS; i++)
        if (AVIFileGetStream(ppFile, &ppAvi, streamtypeVIDEO, i) == 0)
			break;

    if (ppAvi == NULL)
		return E_FAIL;

	m_ppFile = ppFile;
    m_ppAvi = ppAvi;
    
	AVIStreamAddRef(m_ppAvi);
    
    // nastavenie standardneho nazvu exportovaneho BMP 
    i = strVideoFilename.ReverseFind('.');
	
    if (i != -1)
        SetDefaultExportName(strVideoFilename.Left(i));
	else
        SetDefaultExportName(strVideoFilename);

    return 0;
}


// uzavretie videostreamu a suboru
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::CloseFile(){
    long i;
	i = AVIStreamRelease(m_ppAvi);
    i = AVIFileRelease(m_ppFile);
    AVIFileExit(); 

    m_ppAvi = NULL;
    m_ppFile = NULL;
    return 0;
}


// extrakcia snimku podla cisla snimku
// param:	nSampleNum  - cislo snimku, ktory sa ma vyextrahovat
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractFrame(LONG nSampleNum)
{
    PGETFRAME		pGF;
    LPBITMAPINFO	lpBI;
    LPVOID			lpDIBBitmap;
    LONG			nSize;

    // priprava sa na dekompresiu 
    if ((pGF = AVIStreamGetFrameOpen(m_ppAvi, LPBITMAPINFOHEADER(1L))) == NULL)
		return ResultFromScode(AVIERR_NOCOMPRESSOR);

    // vezmeme hlavicku BITMAPY (prve volanie = kolko pamate potrebujeme)
	nSize = 0;
    AVIStreamReadFormat(m_ppAvi, nSampleNum, NULL, &nSize);
	lpBI = (LPBITMAPINFO)(new BYTE[nSize]);
    AVIStreamReadFormat(m_ppAvi, nSampleNum, lpBI, &nSize);

    // a samotne data
    if ((lpDIBBitmap = AVIStreamGetFrame(pGF, nSampleNum)) == NULL) {
	    AVIStreamGetFrameClose(pGF);
	    return ResultFromScode(AVIERR_FILEREAD);
    }

	// pokusime sa zapisat do suboru
	//...
    return 0; 
}


// extrakcia snimku podla cisla snimku
// param:	nSampleNum - cislo snimku, ktory sa ma vyextrahovat
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractFrameBMP(LONG nSampleNum, C4FBitmap* pResultBMP)
{
    PGETFRAME			pGF;
    LPBITMAPINFO		lpBI;
    BYTE*				lpDIBBitmap;
    LONG				nSize;
	BITMAPINFOHEADER*	pBIH;

	// priprava sa na dekompresiu 
    if ((pGF = AVIStreamGetFrameOpen(m_ppAvi, LPBITMAPINFOHEADER(1L))) == NULL)
	    return ResultFromScode(AVIERR_NOCOMPRESSOR);

	// vezmeme hlavicku BITMAPY (prve volanie = kolko pamate potrebujeme)
	nSize = 0;
    AVIStreamReadFormat(m_ppAvi, nSampleNum, NULL, &nSize);
	lpBI = (LPBITMAPINFO)(new BYTE[nSize]);
    AVIStreamReadFormat(m_ppAvi, nSampleNum, lpBI, &nSize);

	// a samotne data
    if ((lpDIBBitmap = (BYTE*)AVIStreamGetFrame(pGF, nSampleNum)) == NULL) {
	    AVIStreamGetFrameClose(pGF);
	    return ResultFromScode(AVIERR_FILEREAD);
    }

    // pokusime sa zapisat do suboru
	pBIH = (BITMAPINFOHEADER*)lpDIBBitmap;
	pResultBMP->SetHeader(pBIH);

	if (pResultBMP->m_pRawData)
		free(pResultBMP->m_pRawData);

	pResultBMP->m_pRawData = (BYTE*)malloc(4 * pBIH->biHeight * pBIH->biWidth * sizeof(BYTE));
	// data su posunute o 13 pixelov
	int nDataOffset = 13;
	int nColPart;
	pResultBMP->m_pHisto->Reset();

	for (int i = 0; i < pBIH->biHeight * pBIH->biWidth; i++) {
		int nRow = i / pBIH->biWidth;
		int nCol = (pBIH->biWidth - 1) - (i % pBIH->biWidth);
		if (nCol < nDataOffset)
			nCol = pBIH->biWidth - (nDataOffset - nCol);
		else
			nCol -= nDataOffset;
		int nPixel = (nRow * pBIH->biWidth) + nCol;
		int B = lpDIBBitmap[3*(pBIH->biHeight *  pBIH->biWidth - i) - 2];
		int G = lpDIBBitmap[3*(pBIH->biHeight *  pBIH->biWidth - i) - 1];
		int R = lpDIBBitmap[3*(pBIH->biHeight *  pBIH->biWidth - i)];

		nColPart = pResultBMP->m_pRawData[4*nPixel] = B;
		pResultBMP->m_pHisto->AddR(nColPart);
		nColPart = pResultBMP->m_pRawData[4*nPixel + 1] = G;
		pResultBMP->m_pHisto->AddG(nColPart);
		nColPart = pResultBMP->m_pRawData[4*nPixel + 2] = R;
		pResultBMP->m_pHisto->AddB(nColPart);
		pResultBMP->m_pRawData[4*nPixel + 3] = 0;
	}

	AVIStreamGetFrameClose(pGF);
	return 0; 
}


// extrakcia snimku podla casu
// param:	nMilisec - casovy udaj, ktory urcuje poziciu snimku v subore
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractTime(LONG nMilisec)
{
	LONG nSampleNum = AVIStreamTimeToSample(m_ppAvi, nMilisec);
	return ExtractFrame(nSampleNum);
}


// extrakcia v urcitom rozsahu snimkov (podla cisiel snimkov)
// param:	nSampleFrom - zaciatok intervalu
//			nSampleTo - koniec intervalu
//			nSampleStep - krok
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractFrameSequence(LONG nSampleFrom, LONG nSampleTo, LONG nSampleStep)
{
	HRESULT hr;
	// predbezne neoptimalizujem, ale pouzivam co uz mam
	for(int i = nSampleFrom; i <= nSampleTo; i += nSampleStep)
		if ((hr = ExtractFrame(i)) != 0)
			return hr;

	return 0;
}


// extrakcia v urcitom rozsahu snimkov (podla casu)
// param:	nMilisecFrom - zaciatok intervalu
//			nMilisecTo - koniec intervalu
//			nTimeStep - krok
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractTimeSequence(LONG nMilisecFrom, LONG nMilisecTo, LONG nTimeStep)
{
	LONG nSampleFrom;
	LONG nSampleTo;
	LONG nSampleStep;

	nSampleFrom = AVIStreamTimeToSample(m_ppAvi, nMilisecFrom);
	nSampleTo = AVIStreamTimeToSample(m_ppAvi, nMilisecTo);
	nSampleStep = (nTimeStep*(nSampleFrom - nSampleTo)) / (nMilisecFrom - nMilisecTo);

	return ExtractFrameSequence(nSampleFrom, nSampleTo, nSampleStep);

}


// extrakcia celeho obsahu AVI-cka
// return:	uspech - 0, inak chybovy kod
HRESULT CAviExtractor::ExtractAll()
{
	AVISTREAMINFO asi;
	AVIStreamInfo(m_ppAvi, &asi, sizeof(asi));
	return ExtractFrameSequence(asi.dwStart, asi.dwStart + asi.dwLength, 1);
}


LPVOID CAviExtractor::ExtractFrameDIB(long nSampleNum)
{
    PGETFRAME		pGF;
    LPBITMAPINFO	lpBI;
	LPVOID			pTemp;
	LPVOID			pResult;
    LONG			nSize;

	// priprava sa na dekompresiu
	pGF = AVIStreamGetFrameOpen(m_ppAvi, 0L);

    // vezmeme hlavicku BITMAPY (prve volanie = kolko pamate potrebujeme)
    nSize = 0;
    AVIStreamReadFormat(m_ppAvi, nSampleNum, NULL, &nSize);
	lpBI = (LPBITMAPINFO)(new BYTE[nSize]);
    AVIStreamReadFormat(m_ppAvi, nSampleNum, lpBI, &nSize);
 
	// a samotne data
    if ((pTemp = AVIStreamGetFrame(pGF, nSampleNum)) == NULL)
	    AVIStreamGetFrameClose(pGF);
	
	pResult = malloc(lpBI->bmiHeader.biSizeImage);
	memcpy(pResult, pTemp, lpBI->bmiHeader.biSizeImage);
	AVIStreamGetFrameClose(pGF);
	return pResult;
}


int CAviExtractor::GetAviLength(void)
{
	AVIFILEINFO*	pFI = new AVIFILEINFO;  
	AVIFileInfo(m_ppFile, pFI, sizeof(AVIFILEINFO));
	return pFI->dwLength;
}
