//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				Frame.cpp
// Ucel suboru:			definicia tried C4FPoint, CHistogram, C4FBitmap a CFrame
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Lucenic / 20.4.2005
//______________________________________________________________________________________________________


#include "StdAfx.h"
#include "frame.h"
#include <complex>

#define WORK_SIZE   1024*32
#define PIX_SIZE 4

/*****************************************************************************/
// modified by Thomas Pintaric, Vienna University of Technology
// BUG in ARToolkit 2.65
// Hardcoded buffer (600*500) is too small for full-size DV-PAL/NTSC resolutions of
// 720x576 and 720x480, respectively. Results in Segmentation faults.
#define HARDCODED_BUFFER_WIDTH  1024
#define HARDCODED_BUFFER_HEIGHT 1024

static ARInt16 l_imageL[HARDCODED_BUFFER_WIDTH * HARDCODED_BUFFER_HEIGHT];
/*****************************************************************************/

static int s_nWorkL[WORK_SIZE];
static int s_nWork2L[WORK_SIZE * 7];
static int s_nWLabel_numL;


CFrame::CFrame(void)
	: m_pBitmap(new C4FBitmap)
{
}


CFrame::~CFrame(void)
{
}


int CFrame::GetPos(void)
{
	return m_nPos - m_pParentSeq->GetSynchroPos();
}


void CFrame::SetBitmap(C4FBitmap* pBitmap)
{	
	m_pBitmap->SetHeader(pBitmap->m_pBmpInfoHdr);
	memcpy(m_pBitmap->m_pRawData, pBitmap->m_pRawData,
		m_pBitmap->m_pBmpInfoHdr->biSizeImage/3 * 4);
	m_pBitmap->m_pHisto->SetHistogram(pBitmap->m_pHisto);
}


C4FBitmap* CFrame::GetBitmap()
{
	return m_pBitmap;
}


// prekresli svoj picture control
void CFrame::RedrawCtrl()
{
}


C4FBitmap::C4FBitmap(void)
	: m_pRawData(NULL)
	, m_pBmpInfoHdr(new BITMAPINFOHEADER)
	, m_pHisto(new CHistogram)
{
}


C4FBitmap::~C4FBitmap(void)
{
}


CBitmap* CFrame::CreateBitmap(void)
{
	CBitmap* pBmp = new CBitmap();
	pBmp->CreateBitmap(m_pBitmap->m_pBmpInfoHdr->biWidth,
		m_pBitmap->m_pBmpInfoHdr->biHeight,
		m_pBitmap->m_pBmpInfoHdr->biPlanes,
		32, m_pBitmap->m_pRawData);

	return pBmp;
}


CHistogram::CHistogram(void)
{
	Reset();
}


CHistogram::~CHistogram(void)
{
}


void C4FBitmap::SetHeader(BITMAPINFOHEADER * pBmpInfoHdr)
{
	m_pBmpInfoHdr->biBitCount = pBmpInfoHdr->biBitCount;
	m_pBmpInfoHdr->biClrImportant = pBmpInfoHdr->biClrImportant;
	m_pBmpInfoHdr->biClrUsed = pBmpInfoHdr->biClrUsed;
	m_pBmpInfoHdr->biCompression = pBmpInfoHdr->biCompression;
	m_pBmpInfoHdr->biHeight = pBmpInfoHdr->biHeight;
	m_pBmpInfoHdr->biPlanes = pBmpInfoHdr->biPlanes;
	m_pBmpInfoHdr->biSize = pBmpInfoHdr->biSize;
	m_pBmpInfoHdr->biSizeImage = pBmpInfoHdr->biSizeImage;
	m_pBmpInfoHdr->biWidth = pBmpInfoHdr->biWidth;
	m_pBmpInfoHdr->biXPelsPerMeter = pBmpInfoHdr->biXPelsPerMeter;
	m_pBmpInfoHdr->biYPelsPerMeter = pBmpInfoHdr->biYPelsPerMeter;
}


CBitmap* CHistogram::GetHistoBMP(void)
{
	CBitmap* pBmp = new CBitmap();
	BYTE pRawData[256*192*4];
	memset(pRawData, 255, 256*192*4);
	int nMaxValueR = 0;
	int nMaxValueG = 0;
	int nMaxValueB = 0;
	for (int i = 0; i < 256; i++) {
		nMaxValueR = max(R[i], nMaxValueR);
		nMaxValueG = max(G[i], nMaxValueG);
		nMaxValueB = max(B[i], nMaxValueB);
	}
	nMaxValueR /= 80;
	nMaxValueG /= 80;
	nMaxValueB /= 80;
	int nPixel;
	for (i = 0; i < 256; i++) {
		nPixel = 4*((191 - R[i]/nMaxValueR)*255 + i);
		pRawData[nPixel] = 255;
		pRawData[nPixel + 1] = 0;
		pRawData[nPixel + 2] = 0;
		pRawData[nPixel + 3] = 0;
		nPixel = 4*((191 - G[i]/nMaxValueG)*255 + i);
		pRawData[nPixel] = 0;
		pRawData[nPixel + 1] = 255;
		pRawData[nPixel + 2] = 0;
		pRawData[nPixel + 3] = 0;
		nPixel = 4*((191 - B[i]/nMaxValueB)*255 + i);
		pRawData[nPixel] = 0;
		pRawData[nPixel + 1] = 0;
		pRawData[nPixel + 2] = 255;
		pRawData[nPixel + 3] = 0;
	}
	pBmp->CreateBitmap(255, 192, 1, 32, pRawData);
	return pBmp;
}


void CHistogram::Reset(void)
{
	memset(R, 0, 256*sizeof(BYTE)*4);
	memset(G, 0, 256*sizeof(BYTE)*4);
	memset(B, 0, 256*sizeof(BYTE)*4);
}


int CHistogram::GetIntensityAt(int nPos)
{
	return R[nPos] + G[nPos] + B[nPos];
}


CFrameSequence* CFrame::GetParent(void)
{
	return m_pParentSeq;
}


void CFrame::SetParent(CFrameSequence* pParent)
{
	m_pParentSeq = pParent;
}


void C4FBitmap::Segmentation(CArray<C4FPoint,C4FPoint&>* pLabels)
{
	//SegmentRegions(pLabels);
	SegmentSimple(pLabels);
}


HRESULT CFrame::Segmentation(void)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*) &m_pParentSeq->m_arLabels.GetAt(m_nPos);
	m_pBitmap->Segmentation(pLabels);

	return E_NOTIMPL;
}


C4FPoint::C4FPoint(void)
	: m_bChecked(false)
{
}


C4FPoint::~C4FPoint(void)
{
}


void CFrame::RenderLabels(CWnd* pWnd, int nSize)
{
	if (m_pParentSeq->m_arLabels.GetCount() <= m_nPos)
		return;

	const int penThickness = 1;
	const int ESIZE = nSize - 2;
	CRect rect;
	pWnd->GetClientRect(&rect);
	
	CDC* pDC = pWnd->GetDC();
	CPen whitePen, redPen;
	CBrush colorBrush;

	redPen.CreatePen(PS_DASH, 1, RGB(255, 0, 0));
	CArray<C4FPoint,C4FPoint&>* pLabels = (CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	
	for (int i = 0; i < pLabels->GetCount(); i++) {
		C4FPoint* pPoint = (C4FPoint*)&pLabels->GetAt(i);
		int x = rect.Width() * pPoint->m_ptBod.x / m_pBitmap->m_pBmpInfoHdr->biWidth;
		int y = rect.Height() * pPoint->m_ptBod.y / m_pBitmap->m_pBmpInfoHdr->biHeight;
		if (pPoint->m_bChecked) {
			pDC->SelectObject(&redPen);
			int R = 255, G = 0, B = 0;
			pDC->MoveTo(x - nSize, y);
			pDC->LineTo(x + nSize, y);
			pDC->MoveTo(x, y - nSize);
			pDC->LineTo(x, y + nSize);
		}
		else {
			int R = (pPoint->m_nGr % 2) * 255,
				G = (pPoint->m_nGr % 4)/2 * 255,
				B = (pPoint->m_nGr % 8)/4 * 255;
			colorBrush.CreateSolidBrush(RGB(R, G, B));
			pDC->SelectObject(&colorBrush);
			whitePen.CreatePen(PS_SOLID, penThickness, RGB(255, 255, 255));
			pDC->SelectObject(&whitePen);
			pDC->Ellipse(x - ESIZE, y - ESIZE, x + ESIZE, y + ESIZE);
		}
		whitePen.DeleteObject();
		colorBrush.DeleteObject();
	}
	redPen.DeleteObject();
}


bool CFrame::LabelOver(int x, int y)
{	
	bool bResult = false;
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	for (int i = 0; i < pLabels->GetCount(); i++) {
		C4FPoint* pPoint = (C4FPoint*) &pLabels->GetAt(i);
		if (x > pPoint->m_ptBod.x - 5 && x < pPoint->m_ptBod.x + 5 &&
			y > pPoint->m_ptBod.y - 5 && y < pPoint->m_ptBod.y + 5) {
			if (!pPoint->m_bChecked)
				bResult = true;
			pPoint->m_bChecked = true;
			break;
		}
		else {
			if (pPoint->m_bChecked)
				bResult = true;
			pPoint->m_bChecked = false;
		}
	}
	for (i++; i < pLabels->GetCount(); i++) {
		C4FPoint* pPoint = (C4FPoint*)&pLabels->GetAt(i);
		pPoint->m_bChecked = false;
	}
	return bResult;
}


void CFrame::LabelDel(void)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	for (int i = 0; i < pLabels->GetCount(); i++) {
		C4FPoint* pPoint = (C4FPoint*) &pLabels->GetAt(i);
		if (!pPoint->m_bChecked)
			continue;
		pLabels->RemoveAt(i);	
		break;
	}
}


void CFrame::LabelAdd(int x, int y)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	C4FPoint ptNew;
	ptNew.m_nGr = 1;
	ptNew.m_ptBod.SetPoint(x, y);
	pLabels->Add(ptNew);
}


bool CFrame::LabelDrag(int x, int y)
{
	bool bResult = false;
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	for (int i = 0; i < pLabels->GetCount(); i++){
		C4FPoint* pPoint = (C4FPoint*)&pLabels->GetAt(i);
		if (!pPoint->m_bChecked)
			continue;
		pPoint->m_ptBod.SetPoint(x, y);
		bResult = true;
	}
	return bResult;
}


void CFrame::LabelSetJoint(int nJointID)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	for (int i = 0; i < pLabels->GetCount(); i++){
		C4FPoint* pPoint = (C4FPoint*)&pLabels->GetAt(i);
		if (!pPoint->m_bChecked)
			continue;
		pPoint->m_nGr = nJointID;
	}
}


int CFrame::LabelGetJointID(void)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_pParentSeq->m_arLabels.GetAt(m_nPos);
	for (int i = 0; i < pLabels->GetCount(); i++){
		C4FPoint* pPoint = (C4FPoint*) &pLabels->GetAt(i);
		if (!pPoint->m_bChecked)
			continue;
		return pPoint->m_nGr;
	}
	return 0;
}


void C4FBitmap::SegmentRegions(CArray<C4FPoint, C4FPoint&>* pLabels)
{
    ARUint8   *pnt;                     /*  image pointer       */
    ARInt16   *pnt1, *pnt2;             /*  image pointer       */
    int       *wk;                      /*  pointer for work    */
    int       wk_max;                   /*  work                */
    int       m,n;                      /*  work                */
    int       i, j, k;                  /*  for loop            */
    int       lxsize, lysize;
    int       poff;
    ARInt16   *l_image;
    int       *work, *work2;
    int       *wlabel_num;

	CArray<CPoint, CPoint&> coor;
	CArray<CPoint, CPoint&> label_group[20];

	l_image = &l_imageL[0];
	work    = &s_nWorkL[0];
	work2   = &s_nWork2L[0];
	wlabel_num = &s_nWLabel_numL;
  
	lxsize = m_pBmpInfoHdr->biWidth;
    lysize = m_pBmpInfoHdr->biHeight;

    pnt1 = &l_image[0];
    pnt2 = &l_image[(lysize-1) * lxsize];
    for(i = 0; i < lxsize; i++)
        *(pnt1++) = *(pnt2++) = 0;

    pnt1 = &l_image[0];
    pnt2 = &l_image[lxsize - 1];
    for(i = 0; i < lysize; i++) {
        *pnt1 = *pnt2 = 0;
        pnt1 += lxsize;
        pnt2 += lxsize;
    }

    wk_max = 0;
    pnt2 = &(l_image[lxsize + 1]);

	pnt = &(m_pRawData[(m_pBmpInfoHdr->biWidth + 1) * PIX_SIZE]);
    poff = PIX_SIZE;
 
	int thresh = 0;
	for (j = 1; j < lysize - 1; j++, pnt += poff*2, pnt2 += 2)
        for (i = 1; i < lxsize - 1; i++, pnt += poff, pnt2++)
            if (*(pnt+1) < *(pnt+2)) {
				pnt1 = &(pnt2[-lxsize]);
				CPoint n_p;
				n_p.SetPoint(i, j);
				coor.Add(n_p);
                if (*pnt1 > 0) {
                    *pnt2 = *pnt1;
                    work2[((*pnt2) - 1)*7 + 0]++;
                    work2[((*pnt2) - 1)*7 + 1] += i;
                    work2[((*pnt2) - 1)*7 + 2] += j;
                    work2[((*pnt2) - 1)*7 + 6] = j;
                }
                else if (*(pnt1 + 1) > 0) {
                    if (*(pnt1 - 1) > 0) {
                        m = work[*(pnt1 + 1) - 1];	// bod za
                        n = work[*(pnt1 - 1) - 1];	// bod pred
						// priradime skumanemu bodu hodnotu bodu,
						// ktory ma mensi labeling
						if (m > n) {
							// prelabelujeme vsetky body, ktore maju
							// hodnotu predch bodu na nizsiu hodnotu
                            *pnt2 = n;
                            wk = &(work[0]);
                            for (k = 0; k < wk_max; k++) {
                                if (*wk == m)
									*wk = n;
                                wk++;
                            }
                        }
                        else if (m < n) {
                            *pnt2 = m;
                            wk = &(work[0]);
                            for (k = 0; k < wk_max; k++) {
                                if (*wk == n)
									*wk = m;
                                wk++;
                            }
                        }
						// maju rovnaky label
                        else
							*pnt2 = m;
                        work2[((*pnt2) - 1)*7 + 0]++;
                        work2[((*pnt2) - 1)*7 + 1] += i;
                        work2[((*pnt2) - 1)*7 + 2] += j;
                        work2[((*pnt2) - 1)*7 + 6] = j;
                    }
                    else if (*(pnt2 - 1) > 0) {
                        m = work[*(pnt1 + 1) - 1];
                        n = work[*(pnt2 - 1) - 1];
                        if (m > n) {
                            *pnt2 = n;
                            wk = &(work[0]);
                            for (k = 0; k < wk_max; k++) {
                                if (*wk == m)
									*wk = n;
                                wk++;
                            }
                        }
                        else if (m < n) {
                            *pnt2 = m;
                            wk = &(work[0]);
                            for (k = 0; k < wk_max; k++) {
                                if (*wk == n)
									*wk = m;
                                wk++;
                            }
                        }
                        else
							*pnt2 = m;
                        work2[((*pnt2) - 1)*7 + 0]++;
                        work2[((*pnt2) - 1)*7 + 1] += i;
                        work2[((*pnt2) - 1)*7 + 2] += j;
                    }
                    else {
                        *pnt2 = *(pnt1 + 1);
                        work2[((*pnt2) - 1)*7 + 0]++;
                        work2[((*pnt2) - 1)*7 + 1] += i;
                        work2[((*pnt2) - 1)*7 + 2] += j;
                        if (work2[((*pnt2) - 1)*7 + 3] > i)
							work2[((*pnt2) - 1)*7 + 3] = i;
                        work2[((*pnt2) - 1)*7 + 6] = j;
                    }
                }
                else if (*(pnt1 - 1) > 0) {
                    *pnt2 = *(pnt1 - 1);
                    work2[((*pnt2) - 1)*7 + 0]++;
                    work2[((*pnt2) - 1)*7 + 1] += i;
                    work2[((*pnt2) - 1)*7 + 2] += j;
                    if (work2[((*pnt2) - 1)*7 + 4] < i)
						work2[((*pnt2) - 1)*7 + 4] = i;
                    work2[((*pnt2) - 1)*7 + 6] = j;
                }
                else if (*(pnt2 - 1) > 0) {
                    *pnt2 = *(pnt2 - 1);
                    work2[((*pnt2) - 1)*7 + 0]++;
                    work2[((*pnt2) - 1)*7 + 1] += i;
                    work2[((*pnt2) - 1)*7 + 2] += j;
                    if (work2[((*pnt2) - 1)*7 + 4] < i)
						work2[((*pnt2) - 1)*7 + 4] = i;
                }
                else {
                    wk_max++;
                    if (wk_max > WORK_SIZE)
						return;
                    work[wk_max - 1] = *pnt2 = wk_max;
                    work2[(wk_max - 1)*7 + 0] = 1;
                    work2[(wk_max - 1)*7 + 1] = i;
                    work2[(wk_max - 1)*7 + 2] = j;
                    work2[(wk_max - 1)*7 + 3] = i;
                    work2[(wk_max - 1)*7 + 4] = i;
                    work2[(wk_max - 1)*7 + 5] = j;
                    work2[(wk_max - 1)*7 + 6] = j;
                }
            }
            else {
                *pnt2 = 0;
				*(pnt) = 0;
				*(pnt+1) = 0;
				*(pnt+2) = 0;
            }

    j = 1;
    wk = &(work[0]);
    for (i = 1; i <= wk_max; i++, wk++)
        *wk = (*wk == i) ? j++ : work[(*wk) - 1];

    *wlabel_num = j - 1;
	int max_rf = -1;
	for (i = 0; i < coor.GetCount(); i++){
		CPoint* in_pt = (CPoint*)&coor.GetAt(i);
		int lri = l_image[in_pt->y * lxsize + in_pt->x];
		int rf = work[lri];
		max_rf = max(rf, max_rf);
		CPoint out_pt;
		out_pt.SetPoint(in_pt->x, in_pt->y);
		if (rf < 20)
			label_group[rf].Add(out_pt);
		else
			break;
	}
	pLabels->RemoveAll();
	for (i = 0; i < max_rf; i++)
		for (j = 0; j < label_group[i].GetCount(); j++) {
			C4FPoint center;
			center.m_ptBod.SetPoint(0, 0);
			CPoint* in_pt = (CPoint*)&label_group[i].GetAt(j);
			center.m_ptBod.x += in_pt->x;
			center.m_ptBod.y += in_pt->y;
			pLabels->Add(center);
		}
}


void C4FBitmap::SegmentSimple(CArray<C4FPoint, C4FPoint&>* pLabels)
{
	CArray<C4FPoint, C4FPoint&> arTemp;
	int thresh = 0;
	pLabels->RemoveAll();

	for (int i = m_pBmpInfoHdr->biWidth + 1; i < (m_pBmpInfoHdr->biHeight - 1) * m_pBmpInfoHdr->biWidth; i++) {
		if (m_pRawData[4*i + 1] >= m_pRawData[4*i + 2])
			continue;
		C4FPoint ptNew;
		ptNew.m_ptBod.SetPoint(i % m_pBmpInfoHdr->biWidth, i / m_pBmpInfoHdr->biWidth);
		ptNew.m_nIntensity = m_pRawData[4*i + 2];
		arTemp.Add(ptNew);
    }
	
	int m_nGr = 0;
	for (i = 0; i < arTemp.GetCount(); i++) {
		C4FPoint* pA = (C4FPoint*)&arTemp.GetAt(i);
		C4FPoint* pB;
		for (int j = 0; j < i; j++) {
			pB = (C4FPoint*) &arTemp.GetAt(j);
			double fDist = sqrt(pow(abs(pA->m_ptBod.x - pB->m_ptBod.x), 2) + pow(abs(pA->m_ptBod.y - pB->m_ptBod.y), 2));
			if (fDist < 5)
				break;
		}
		if (j < i)
			pA->m_nGr = pB->m_nGr;
		else
			pA->m_nGr = m_nGr++;
	}
	
	for (i = 0; i < arTemp.GetCount(); i++) {
		C4FPoint* pA = (C4FPoint*)&arTemp.GetAt(i);
		if (pA->m_nGr >= pLabels->GetCount()) {
			C4FPoint ptNew;
			ptNew.m_nIntensity = 0;
			ptNew.m_ptBod.SetPoint(0, 0);
			pLabels->Add(ptNew);
		}
		C4FPoint* pPt = (C4FPoint*) &pLabels->GetAt(pA->m_nGr);
		if (pA->m_nIntensity > pPt->m_nIntensity) {
			pPt->m_nIntensity = pA->m_nIntensity;
			pPt->m_ptBod.SetPoint(pA->m_ptBod.x, pA->m_ptBod.y);
			pPt->m_nGr = 1;
		}
	}
}
