//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				Frame.h
// Ucel suboru:			deklaracia tried C4FPoint, CHistogram, C4FBitmap a CFrame
//						deklaracia s definiciou triedy CLabelArray
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Lucenic / 20.4.2005
//______________________________________________________________________________________________________


#ifndef __frame_h__
#define __frame_h__

#include "FrameSequence.h"

#pragma once

typedef char              ARInt8;
typedef short             ARInt16;
typedef int               ARInt32;
typedef unsigned char     ARUint8;
typedef unsigned short    ARUint16;
typedef unsigned int      ARUint32;

//reprezentuje detekovany klb
class C4FPoint
{
public:
	C4FPoint(void);
	~C4FPoint(void);
	C4FPoint(const C4FPoint& pt) {
		m_ptBod.SetPoint(pt.m_ptBod.x, pt.m_ptBod.y);
		m_bChecked = pt.m_bChecked;
		m_nGr = pt.m_nGr;
		m_nIntensity = pt.m_nIntensity;
	}
	C4FPoint& operator=(const C4FPoint& pt) {
		m_ptBod.SetPoint(pt.m_ptBod.x, pt.m_ptBod.y);
		m_bChecked = pt.m_bChecked;
		m_nGr = pt.m_nGr;
		m_nIntensity = pt.m_nIntensity;
		return *this;
	}
	//suradnice klbu
	CPoint	m_ptBod;
	//flag na zistenie onPress satusu mysi na tento objekt
	bool	m_bChecked;
	//udava identifikartor klbu, ku ktoremu bod prislucha
	int		m_nGr;
	//hodnota intenzity klbu, vyuzije sa pri urcovani stredu znacky
	int		m_nIntensity;
};

//zoznam vetkch pol znaiek sekvencie
class CLabelArray
{
public:
	CLabelArray(const CArray<C4FPoint,C4FPoint&>& arPoints){
		for (int i = 0; i < arPoints.GetCount(); i++) {
			C4FPoint pt = arPoints.GetAt(i);
			m_arLabels.Add(pt);
		}
	}
	CLabelArray& operator=(const CLabelArray& arLabels){
		for (int i = 0; i < arLabels.m_arLabels.GetCount(); i++) {
			C4FPoint pt = arLabels.m_arLabels.GetAt(i);
			m_arLabels.Add(pt);
		}
		return *this;
	}
	//pole bodov, reprezentujucich klb
	CArray<C4FPoint,C4FPoint&> m_arLabels;

	CLabelArray(void) {}
	~CLabelArray(void) {}
};

//trieda hisstogramu pre snimok
class CHistogram
{
private:
	//pocetnosty hodnot jednotlivych zloziek farby
	int R[256];
	int G[256];
	int B[256];

public:
	CHistogram(void);
	~CHistogram(void);
	//zistvanie suctu RGB pre konkretnu hodnotu zlozky
	//3 * pocet bodov danej intenzity v ramci obrazku
	int GetIntensityAt(int nPos);
	//zvysenie poctu bodov danej hodnoty farebnej zlozky
	void AddR(int nR) { R[nR]++; }
	void AddG(int nG) { G[nG]++; }
	void AddB(int nB) { B[nB]++; }
	//kopirovanie hisstogramu
	void SetHistogram(CHistogram* pHisto) {
		memcpy(R, pHisto->R, 256*4);
		memcpy(G, pHisto->G, 256*4);
		memcpy(B, pHisto->B, 256*4);
	}
	//vytvorenie bitmapy na zaklade histogramu
	CBitmap* GetHistoBMP(void);
	void Reset(void);
};

//trieda reprezentujca bitmapu priraden k snmku
class C4FBitmap
{
public:
	C4FBitmap(void);
	~C4FBitmap(void);
	//bitmapa vo formate BGRA
	BYTE*				m_pRawData; 
	//hlavicka bitmapy
	BITMAPINFOHEADER*	m_pBmpInfoHdr;
	//histogram odpovedajuci bitmape
	CHistogram*			m_pHisto;
	//nastavenie hlavicky
	void SetHeader(BITMAPINFOHEADER* pBmpInfoHdr);
	//segmentacia bitmapy
	//vystup: nstavene pole detekovanych znaciek
	void Segmentation(CArray<C4FPoint,C4FPoint&> *pLabels);
	//segemntacia cez suvisle oblasti
	//!!!momentalne nepouzita
	void SegmentRegions(CArray<C4FPoint, C4FPoint&>* pLabels);
	//segmentacia cez intenzitu bodov a ich vzajomne vzdialenosti
	void SegmentSimple(CArray<C4FPoint, C4FPoint&>* pLabels);
};


class CFrameSequence;
//reprezentcia jednho snmku v rmci sekvencie videa
class CFrame
{
private:
	// bitmapa, ktora prislucha ku frame-u danemu hodnotou nPos
	C4FBitmap*		m_pBitmap;
	//parent sekvencia, z ktorej pochadza snimka
	CFrameSequence*	m_pParentSeq;
	// pozicia v ramci sekvencie
	int	m_nPos;
	//ID resource komponentov dialogu
	//pole kam sa zobrazi snimka
	int m_nIDPictureCtrl;
	//ram pola
	int m_nIDFrameNumCtrl;
	//nepouzivane
	int m_nIDHistoCtrl;
public:
	CFrame(void);
	//copy constructor
	CFrame(const CFrame& cp) {
		m_nPos = cp.m_nPos;
		m_pBitmap = cp.m_pBitmap;
		m_nIDPictureCtrl = cp.m_nIDPictureCtrl;
		m_pParentSeq = cp.m_pParentSeq;
	};
	~CFrame(void);
	//nstavenie pozicie
	void SetPos(int nPos) { m_nPos = nPos; }
	//vratenie pozicie
	int GetPos();
	//priradenie resource komponentov ku snimke
	void AssignPicCtrl(int nID) { m_nIDPictureCtrl = nID; }
	int GetPicCtrl(void) { return m_nIDPictureCtrl; }
	void AssignHistoCtrl(int nID) { m_nIDHistoCtrl = nID; }
	int GetHistoCtrl(void) { return m_nIDHistoCtrl; }
	void AssignFrNumCtrl(int nID) { m_nIDFrameNumCtrl = nID; }
	int GetFrNumCtrl(void) { return m_nIDFrameNumCtrl; }

	CFrame& operator=(const CFrame& cp){
		this->m_nPos = cp.m_nPos;
		this->m_pBitmap = cp.m_pBitmap;
		this->m_nIDPictureCtrl = cp.m_nIDPictureCtrl;
		this->m_pParentSeq = cp.m_pParentSeq;
		return *this;
	}
	//nastavenie Bitmapy
	//vstup: *CBitmap
	//z CBitmap sa vytvori C4FBitmap, nastavi sa histogram a hlavicka
	void SetBitmap(C4FBitmap* pBitmap);
	C4FBitmap* GetBitmap();
	//z C4FBitmap vytvori CBitmap vhodnu na vykreslenie
	//treba dorobit konverziu z fotmatov (momentalen funguje na zobrazenie 24bitovch bitmap
	CBitmap* CreateBitmap(void); 
	//vyvolanie prekreslenia
	void RedrawCtrl();
	//ziskanie a nastavenie rodicovskej sekvencie
	CFrameSequence* GetParent(void);
	void SetParent(CFrameSequence* pParent);
	//vyvolanie segemntavie snimku
	//vyvola segmentovanie m_pBitmap
	HRESULT Segmentation(void);
	//vykreslovanie znaciek
	void RenderLabels(CWnd* pWnd, int nSize);
	//nastavenie priznaku pre znacku so suradnizami v stvorci 10x10px okolo bodu x, y
	bool LabelOver(int x, int y);
	//presun oznacenej znacky na poziciu x, y
	bool LabelDrag(int x, int y);
	//pridanie novej znacky na poziciu x, y
	void LabelAdd(int x, int y);
	//zrusenie znacky, ktora ma nastaveny priznak udavajuci ze je under mouse
	void LabelDel(void);
	//priradenie ozancenej znacky ku klby s ID nJointID
	void LabelSetJoint(int nJointID);
	int LabelGetJointID(void);
};

#endif // __frame_h__
