//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				FrameSequence.cpp
// Ucel suboru:			definicia triedy CFrameSequence
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Argalas / 7.5.2005
//______________________________________________________________________________________________________


#include "StdAfx.h"
#include "FrameSequence.h"
#include "VideoSQC.h"
#include "VideoSQCDlg.h"
#include "FrameDlg.h"

extern CVideoSQCApp theApp;
extern std::ofstream output;
extern std::ifstream input;


CFrameSequence::CFrameSequence(void)
{
	InitParam();
}


CFrameSequence::CFrameSequence(int nFrameCount)
{
	for (int i = 0; i < nFrameCount; i++) {
		CFrame frame;
		frame.SetParent(this);
        m_arChildren.Add(frame);
	}
	InitParam();
}


CFrameSequence::~CFrameSequence(void)
{
}


void CFrameSequence::InitParam(void)
{
	m_pStream = NULL;
	m_nSynchroPos = 0;
}


HRESULT CFrameSequence::Next()
{
	C4FBitmap*	pBmp;
	if (GetStreamLength() - 3 <= m_nCurPos - m_nSynchroPos)
		return S_FALSE;

	m_nCurPos++;
	for (int i = 0; i < FRAME_COUNT - 1; i++) {
        CFrame* pFrame = (CFrame*)&m_arChildren.GetAt(i);
        CFrame* pNextFrame = (CFrame*)&m_arChildren.GetAt(i + 1);
		pBmp = pNextFrame->GetBitmap();
		pFrame->SetBitmap(pBmp);
		pFrame->SetPos(m_nCurPos + i);
	}
    CFrame* pFrame = (CFrame*)&m_arChildren.GetAt(FRAME_COUNT - 1);
	pBmp = pFrame->GetBitmap();
	pFrame->SetPos(m_nCurPos + FRAME_COUNT - 1);
	m_pStream->ExtractFrameBMP(m_nCurPos + FRAME_COUNT - 1, pBmp);
	
	return 0;
}


HRESULT CFrameSequence::Prev()
{
	C4FBitmap* pBmp;
	if (m_nCurPos - m_nSynchroPos == 0)
		return S_FALSE;

	m_nCurPos--;
	for (int i = FRAME_COUNT - 1; i > 0; i--) {
        CFrame* pFrame = (CFrame*)&m_arChildren.GetAt(i);
        CFrame* pPrevFrame = (CFrame*)&m_arChildren.GetAt(i-1);
		pBmp = pPrevFrame->GetBitmap();
		pFrame->SetBitmap(pBmp);
		pFrame->SetPos(m_nCurPos + i);
	}
    CFrame* pFrame = (CFrame*)&m_arChildren.GetAt(0);
	pBmp = pFrame->GetBitmap();
	pFrame->SetPos(m_nCurPos);
	m_pStream->ExtractFrameBMP(m_nCurPos, pBmp);

	return 0;
}


void CFrameSequence::InitFrames(int* pCtrlIDs, int nCount)
{
	m_nIDFirstPictureHolder = pCtrlIDs[0];

	for (int i = 0; i < nCount; i++) {
		CFrame* pCurFrame = (CFrame*)&m_arChildren.GetAt(i);
		pCurFrame->AssignPicCtrl(pCtrlIDs[i]);
	}
	for (; i < nCount*2; i++) {
		CFrame* pCurFrame = (CFrame*)&m_arChildren.GetAt(i % 4);
		pCurFrame->AssignFrNumCtrl(pCtrlIDs[i]);
	}
	m_nIDMainGroupBox = pCtrlIDs[i];
	m_nIDSlider = pCtrlIDs[i + 1];
	m_nIDPlayBtn = pCtrlIDs[i + 2];
}


void CFrameSequence::SetStream(CAviExtractor* pStream)
{
	m_pStream = pStream;
	m_nSynchroPos = 0;
	// iba test	
	m_nStreamLength = m_pStream->GetAviLength();	
	m_arLabels.RemoveAll();
	for (int i = 0; i < m_nStreamLength+1; i++) {
		CLabelArray new_la;
		m_arLabels.Add(new_la);
	}
}


bool CFrameSequence::IsLoaded(void)
{
	return (m_pStream != NULL);
}


HRESULT CFrameSequence::GotoFrame(int nFrameNum)
{
	C4FBitmap* pBmp;
	if (GetStreamLength() - 3 <= nFrameNum)
		return S_FALSE;

	m_nCurPos = nFrameNum + m_nSynchroPos;
	for (int i = 0; i < FRAME_COUNT; i++) {
        CFrame* pFrame = (CFrame*)&m_arChildren.GetAt(i);
		pBmp = pFrame->GetBitmap();
		pFrame->SetPos(nFrameNum + m_nSynchroPos + i);
		m_pStream->ExtractFrameBMP(m_nCurPos + i, pBmp);
	}
	return 0;
}


int CFrameSequence::GetStreamLength(void)
{
	return m_nStreamLength;
}


bool CFrameSequence::IsLastFrame(void)
{
	return (GetStreamLength() - 3 <= m_nCurPos - m_nSynchroPos);
}


HRESULT CFrameSequence::FindSamples(int nStartFrame /*= 0*/)
{
	GotoFrame(nStartFrame);
	float fPercOfConfid = 0.0;
	do {
		fPercOfConfid = CompareFrames(1,2);
		Next();
	} while(fPercOfConfid > 0.5 && !IsLastFrame());
	
	if (!IsLastFrame()) {
		Next();
		return S_OK;
	}
	else {
		GotoFrame(nStartFrame);
		return S_FALSE;
	}
}


float CFrameSequence::CompareFrames(int nFrameA, int nFrameB)
{
	CFrame* pFrameA = (CFrame*)&m_arChildren.GetAt(nFrameA);
	CFrame* pFrameB = (CFrame*)&m_arChildren.GetAt(nFrameB);
	float fPercOfConfid = 0.0;
	for(int i = 0; i < 255; i++) {
        int nIntenistyA = pFrameA->GetBitmap()->m_pHisto->GetIntensityAt(i);
        int nIntenistyB = pFrameB->GetBitmap()->m_pHisto->GetIntensityAt(i);
		float fLower = (float)min(nIntenistyA, nIntenistyB) + 1;
		float fHigher =(float)max(nIntenistyA, nIntenistyB) + 1;
		fPercOfConfid += fLower / fHigher;
	}
	return fPercOfConfid / 255;
}


int CFrameSequence::Lock(int nLength)
{
	m_nSynchroPos = m_nCurPos;
 	m_nStreamLength = nLength;
	return 0;
}


int CFrameSequence::Unlock()
{
	m_nSynchroPos = 0;
 	m_nStreamLength = m_pStream->GetAviLength();
	return 0;
}


int CFrameSequence::GetSynchroPos()
{
	return m_nSynchroPos;
}


HRESULT CFrameSequence::Segmentation(void)
{
	CFrame* pFrame = new CFrame();
	pFrame->SetParent(this);
	C4FBitmap* pBmp = pFrame->GetBitmap();
	CProgressCtrl* pProgressBar = (CProgressCtrl*)theApp.m_pMainWnd->GetDlgItem(IDC_PROGRESS);
	pProgressBar->SetRange(0, m_nStreamLength);
	pProgressBar->SetPos(0);
	pProgressBar->ShowWindow(SW_SHOW);
	for (int i = 0; i < m_nStreamLength + 1; i++) {
		m_pStream->ExtractFrameBMP(i, pBmp);
		pFrame->SetPos(i);
		pFrame->Segmentation();
		pProgressBar->SetPos(i);
	}
	pProgressBar->ShowWindow(SW_HIDE);
	return E_NOTIMPL;
}


void CFrameSequence::Show(int nFrameID)
{
	CFrameDlg* pDlg = new CFrameDlg();
	pDlg->m_pFrame = (CFrame*)&m_arChildren.GetAt(nFrameID);
	pDlg->DoModal();
	CVideoSQCDlg* pWnd = (CVideoSQCDlg*)theApp.m_pMainWnd;
	pWnd->Invalidate(0);
}


HRESULT CFrameSequence::JointsSort(void)
{
	CArray<C4FPoint,C4FPoint&>* pLabels =
		(CArray<C4FPoint,C4FPoint&>*)&m_arLabels.GetAt(m_nCurPos);
	
	for (int i = m_nCurPos + 1; i < m_nStreamLength + 1; i++){
		CArray<C4FPoint,C4FPoint&>* pLabelsCur =
			(CArray<C4FPoint,C4FPoint&>*)&m_arLabels.GetAt(i);
		CArray<C4FPoint,C4FPoint&>* pLabelsPrev =
			(CArray<C4FPoint,C4FPoint&>*)&m_arLabels.GetAt(i - 1);
		for (int j = 0; j < pLabelsCur->GetCount(); j++){
			C4FPoint* ptCur = (C4FPoint*)&pLabelsCur->GetAt(j);
			double fMinDist = 1000000.0;
			for (int k = 0; k < pLabelsPrev->GetCount(); k++){
				C4FPoint* ptPrev = (C4FPoint*) &pLabelsPrev->GetAt(k);
				double fDist = sqrt(pow(abs(ptCur->m_ptBod.x - ptPrev->m_ptBod.x), 2)
								+ pow(abs(ptCur->m_ptBod.y - ptPrev->m_ptBod.y), 2));
				if (fDist >= fMinDist)
					continue;
				fMinDist = fDist;
				ptCur->m_nGr = ptPrev->m_nGr;
			}
		}
	}
	return E_NOTIMPL;
}


HRESULT CFrameSequence::JointsSave(int format)
{
	output << 25 ;
	output << '\n';
	output << m_nStreamLength + 1;
	output << '\n';
	output << 1;
	output << "\n\n";

	for (int i = 0; i < m_nStreamLength + 1; i++) {
		CArray<C4FPoint,C4FPoint&>* pLabels =
			(CArray<C4FPoint,C4FPoint&>*)&m_arLabels.GetAt(i);
		if(format == BBS_FORMAT)
			output << "FR " << i << " LC " << long(pLabels->GetCount()) << "\n";
		int repeat = 1;
		if(format == BBD_FORMAT) repeat++;
		for(int r = 0; r < repeat; r++){
			for (int j = 0; j < pLabels->GetCount(); j++){
				C4FPoint* pt = (C4FPoint*)&pLabels->GetAt(j);
				if(format == BBS_FORMAT) output << pt->m_nGr << ' ';
				output << pt->m_ptBod.x << ' ';
				output << pt->m_ptBod.y << ' ';
				output << '0' << ' ';
			}
			output << "\n";
		}
		output << "\n";
	}
	return E_NOTIMPL;
}

HRESULT CFrameSequence::JointsLoad()
{
	char str[10];
	char ignore[10];
	int frame_num;
	int label_cnt;
	input >> ignore;
	input >> ignore;
	input >> ignore;
	CArray<C4FPoint,C4FPoint&>* pLabels = NULL; 
	
	while (input >> str) {
		if (strcmp("SEQ",str) == 0) {
			using namespace std;
			input.seekg(-3, ios_base::cur);
			return S_OK;
		}
		if (strcmp("FR",str) == 0) {
			input >> frame_num;
			if(frame_num > m_arLabels.GetCount() - 1) {
				AfxMessageBox("Labels overflow - tail ignored");
				return S_FALSE;
			}
			pLabels = (CArray<C4FPoint,C4FPoint&>*)&m_arLabels.GetAt(frame_num);
		}
		if (strcmp("LC",str)  == 0) {
			input >> label_cnt;
			if (!pLabels) {
				AfxMessageBox("File format error");
				return S_FALSE;
			}
			pLabels->RemoveAll();
			for(int i = 0; i < label_cnt; i++){
				C4FPoint new_point;
				new_point.m_bChecked = false;
				input >> new_point.m_nGr;
				input >> new_point.m_ptBod.x;
				input >> new_point.m_ptBod.y;
				input >> ignore;
				pLabels->Add(new_point);
			}
		}
	}

	return E_NOTIMPL;
}
