//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				FrameSequence.h
// Ucel suboru:			deklaracia triedy CFrameSequence
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Lucenic / 20.4.2005
//______________________________________________________________________________________________________


#ifndef __frameseq_h__
#define  __frameseq_h__

#include "AviExtractor.h"
#include <afxtempl.h>
#include "Frame.h"

#pragma once

class CFrame;
class CAviExtractor;
class C4FPoint;
class CLabelArray;

//reprezentuje aktualne zobrazovanu sekvenciu
class CFrameSequence
{
private:
	// rodicovsky video stream, z ktoreho sa budu tahat data
	CAviExtractor*	m_pStream;
	//aktualna pozicia v ramci streamu
	int		m_nCurPos;
	//pozicia synchronizacnej snimky
	int		m_nSynchroPos;
	//dlzka streamu
	int		m_nStreamLength;
	//pole poli znaciek pre vsetky snimky
	CArray<CLabelArray,CLabelArray&> m_arLabels;
	
	friend class CFrame;

public:
	// IDcka vybranych resourcov
	int m_nIDFirstPictureHolder;
	int m_nIDPlayBtn;
	int m_nIDMainGroupBox;
	int m_nIDSlider;
	void InitParam(void);
	// zoznam frameov, ktore tvoria sekvenciu
	CArray<CFrame, CFrame&> m_arChildren;
	//presun na dalsiu snimku
	HRESULT Next();
	//presun na predosluiu snimku
	HRESULT Prev();
	//priradenie zdrojoveho suboru
	void SetStream(CAviExtractor* pStream);
	//inicializacia sekvencie, zhrna priradenie resource komponentov k snimkom
	void InitFrames(int* pCtrlIDs, int nCount);

	CFrameSequence(void);
	CFrameSequence(int nFrameCount);
	~CFrameSequence(void);
	//vrati true ak je sekvencia nacitana
	bool IsLoaded(void);
	//vrati aktualnu poziciu
	//zavisi od toho ci boli snimky synchronizovane
	int GetCurPos() const { return (m_nSynchroPos == -1) ? m_nCurPos : m_nCurPos - m_nSynchroPos; }
	//prechod na konkretnu snimku
	HRESULT GotoFrame(int nFrameNum);
	//vrati dlzku
	int GetStreamLength(void);
	//zistenie ci sme na konci sekvencie
	bool IsLastFrame(void);
	//metoda na najdenei synchronizacnych snimkov
	HRESULT FindSamples(int nStartFrame = 0);
	//porovna histogramy snimkov
	//vstup: poadove cisla snimok
	//vystup: miera zhody kistogramov
	float CompareFrames(int nFrameA, int nFrameB);
	//spojenie prehravania sekvencii
	int Lock(int nLength);
	//odomknutie prehravania sekvencii
	int Unlock();
	//zisti poziciu synchronizacnej snimky
	int GetSynchroPos(void);
	//segemtacia
	HRESULT Segmentation(void);
	//vyvola zobrazene konkretnej snimky
	void Show(int nFrameID);
	//korespodencia
	HRESULT JointsSort(void);
	//ulozenie snimok v zvolenom formate na zvolene miesto
	HRESULT JointsSave(int format);
	//nacitanie z formatu bbs
	HRESULT JointsLoad();
};

#endif // __frameseq_h__
