//______________________________________________________________________________________________________
// Projekt:				Vizualizacia a analyza chodze cloveka
//						Timovy projekt 2004/5 - tim 4F (Four Fit Feet of FIIT)
//						FIIT STU, Bratislava
//------------------------------------------------------------------------------------------------------
// Subor:				SequenceHolder.cpp
// Ucel suboru:			definicia triedy CSequenceHolder
// Datum vytvorenia:	5.4.2005
// Vytvoril:			Peter Argalas
// Naposledy zmenil:	Lucenic / 12.05.2005
//______________________________________________________________________________________________________


#include "StdAfx.h"
#include "sequenceholder.h"
#include "AVIStream2BMP.h"

#include <iostream>

std::ifstream input;
std::ofstream output;


CSequenceHolder::CSequenceHolder(CVideoSQCDlg* pDlg)
	: m_pDlg(pDlg)
{
}


void CSequenceHolder::InitSeq(int nSeqID, int* pCtrlIDs)
{
	CFrameSequence* pSeq = new CFrameSequence(4);
	pSeq->InitFrames(pCtrlIDs, 4);
	m_pSeq[nSeqID - 1] = pSeq;
	m_nNumSeq = 0;
}


CSequenceHolder::~CSequenceHolder(void)
{
}


HRESULT CSequenceHolder::SetStream(int nSeqID, CString strFileName)
{
	// window handle of dialog box 
	CFrameSequence*	pSeq = m_pSeq[nSeqID - 1];
	CAviExtractor*	aviStream = new CAviExtractor(strFileName);
	pSeq->SetStream(aviStream);
	
	int length = pSeq->GetStreamLength();
	GetSlider(pSeq)->SetRange(0, length - FRAME_COUNT);
    strFileName.Format("Stream 1, Length: %d frames, Synchronized: No", length);
	
	// upravime dialog
	pSeq->GotoFrame(0);
	m_pDlg->GetDlgItem(pSeq->m_nIDMainGroupBox)->SetWindowText(strFileName);
	m_nNumSeq=((m_nNumSeq+1)>SEQ_COUNT)?SEQ_COUNT:(m_nNumSeq+1);
	Invalidate(-1);

	return S_OK;
}


HRESULT CSequenceHolder::Redraw(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (pSeq->m_nIDFirstPictureHolder != nSeqID)
			continue;
		if (!pSeq->IsLoaded())
			continue;
		for (int j = 0; j < FRAME_COUNT; j++){
			CFrame* pFrame = (CFrame*)&pSeq->m_arChildren.GetAt(j);
			m_pDlg->PictureRedraw(pFrame);
		}
	}
	return S_OK;
}


HRESULT CSequenceHolder::Next(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	pSeq->Next();
	GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
	Invalidate(nSeqID);

	return S_OK;
}


HRESULT CSequenceHolder::Prev(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	pSeq->Prev();
	GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
	Invalidate(nSeqID);

	return S_OK;
}


HRESULT CSequenceHolder::Play(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	CString strStatus;
	m_pDlg->GetDlgItem(pSeq->m_nIDPlayBtn)->GetWindowText(strStatus);
	if (strStatus == "Play") {
		m_pDlg->m_nTimers[nSeqID] = m_pDlg->SetTimer(nSeqID, 40, 0);
		m_pDlg->GetDlgItem(pSeq->m_nIDPlayBtn)->SetWindowText("Pause");
	}
	else {
		m_pDlg->GetDlgItem(pSeq->m_nIDPlayBtn)->SetWindowText("Play");
		m_pDlg->KillTimer(m_pDlg->m_nTimers[nSeqID]);
	}

	return S_OK;
}


HRESULT CSequenceHolder::Reset(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	pSeq->GotoFrame(0);
	GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
	Invalidate(nSeqID);

	return S_OK;
}


HRESULT CSequenceHolder::GoToPos(int nSeqID, int nPos)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	pSeq->GotoFrame(nPos);
	GetSlider(pSeq)->SetPos(pSeq->GetCurPos());	
	Invalidate(nSeqID);

	return S_OK;
}


HRESULT CSequenceHolder::OnTimer(int nSeqID)
{
	if (!m_nNumSeq)
		return S_FALSE;

	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return S_FALSE;

	if (pSeq->IsLastFrame())
		Reset(nSeqID);
	else
		Next(nSeqID);
	Invalidate(nSeqID);

	return S_OK;
}


HRESULT CSequenceHolder::FindSamples()
{
	int nCurPos[SEQ_COUNT];

	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (!pSeq->IsLoaded())
			return S_FALSE;
		nCurPos[i] = pSeq->GetCurPos();
	}
	for(int i = 0; i< SEQ_COUNT; i++){
		CFrameSequence* pSeq = m_pSeq[i];
		if (pSeq->FindSamples() == S_OK) {
			GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
			continue;
		}
		GoToPos(i + 1, nCurPos[i]);
		CString str;
		str.Format("Stream %d: No synchronization point found!\nContinue?", i + 1);
		if (AfxMessageBox(str, MB_YESNO) != IDYES)
			return S_FALSE;
	}
	Invalidate(-1);

	return S_OK;
}


HRESULT CSequenceHolder::Lock()
{
	int nMinLength = m_pSeq[0]->GetStreamLength() - m_pSeq[0]->GetCurPos();
	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (!pSeq->IsLoaded())
			return S_FALSE;
		nMinLength = min(nMinLength, pSeq->GetStreamLength() - pSeq->GetCurPos());
	}
	for (int i = 0; i < SEQ_COUNT; i++){
		CFrameSequence* pSeq = m_pSeq[i];
		pSeq->Lock(nMinLength);
		GetSlider(pSeq)->SetRange(0, pSeq->GetStreamLength() - FRAME_COUNT);
		GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
		CString str;
		str.Format("Stream 1, Length: %d frames, Synchronized: Yes", pSeq->GetStreamLength());
		m_pDlg->GetDlgItem(pSeq->m_nIDMainGroupBox)->SetWindowText(str);
	}
	Invalidate(-1);

	return S_OK;
}


HRESULT CSequenceHolder::Unlock(void)
{
	for (int i = 0; i < SEQ_COUNT; i++){
		CFrameSequence* pSeq = m_pSeq[i];
		pSeq->Unlock();
		GetSlider(pSeq)->SetRange(0, pSeq->GetStreamLength() - FRAME_COUNT);
		GetSlider(pSeq)->SetPos(pSeq->GetCurPos());
		CString str;
		str.Format("Stream 1, Length: %d frames, Synchronized: No", pSeq->GetStreamLength());
		m_pDlg->GetDlgItem(pSeq->m_nIDMainGroupBox)->SetWindowText(str);
	}
	Invalidate(-1);

	return S_OK;
}


CSliderCtrl* CSequenceHolder::GetSlider(CFrameSequence* pSeq)
{
	return (CSliderCtrl*)m_pDlg->GetDlgItem(pSeq->m_nIDSlider);
}


void CSequenceHolder::Invalidate(int nSeqID)
{
	if (nSeqID == -1)
		for (int i = 0 ; i < SEQ_COUNT; i++) {
			CFrameSequence* pSeq = m_pSeq[i];
			m_pDlg->GetDlgItem(pSeq->m_nIDFirstPictureHolder)->Invalidate(0);
		}
	else {
		CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
		m_pDlg->GetDlgItem(pSeq->m_nIDFirstPictureHolder)->Invalidate(0);
	}
}


HRESULT CSequenceHolder::Segmentation(void)
{
	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (!pSeq->IsLoaded())
			continue;
		CString message;
		message.Format("Segment stream no. %d?", i+1);
		if(AfxMessageBox(message, MB_YESNO, 0) == IDNO) continue;
		pSeq->Segmentation();
	}
	Invalidate(-1);

	return S_OK;
}


void CSequenceHolder::Show(int nSeqID, int nFrameID)
{
	CFrameSequence* pSeq = m_pSeq[nSeqID - 1];
	if (!pSeq->IsLoaded())
		return;
	pSeq->Show(nFrameID);
}


HRESULT CSequenceHolder::JointsSort(void)
{
	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (!pSeq->IsLoaded())
			continue;
		CString message;
		message.Format("Sort stream no. %d?", i+1);
		if(AfxMessageBox(message, MB_YESNO, 0) == IDNO) continue;
		pSeq->JointsSort();
	}
	Invalidate(-1);

	return S_OK;
}


HRESULT CSequenceHolder::JointsSave(CString filename, int format)
{
	using namespace std;
	if (format == BBS_FORMAT)
		output.open(filename, ios_base::out | ios_base::trunc);
	for (int i = 0; i < SEQ_COUNT; i++) {
		CFrameSequence* pSeq = m_pSeq[i];
		if (!pSeq->IsLoaded())
			continue;
		if (format == BBD_FORMAT) {
			CString newFilename;
			newFilename.Format("%s%d.bbd", filename, i+1);
			output.open(newFilename, ios_base::out | ios_base::trunc);
		}
		if (format == BBS_FORMAT) output << "SEQ " << i+1 << "\n";
		pSeq->JointsSave(format);
		if (format == BBD_FORMAT) output.close();
	}
	if (format == BBS_FORMAT) output.close();
	Invalidate(-1);

	return S_OK;
}


HRESULT CSequenceHolder::JointsLoad(const char* filename)
{
	using namespace std;
	input.clear();
	input.open(filename, ios_base::in);
	char str[10];

	while (input >> str) {
		if (strcmp("SEQ",str)) {
			AfxMessageBox("Unrecognized file format");
			input.close();
			return S_FALSE;
		}
		int nSeqID;
		input >> nSeqID;
		ASSERT(nSeqID <= SEQ_COUNT);
		CFrameSequence* pSeq = m_pSeq[nSeqID-1];

		if (!pSeq->IsLoaded())
			break;
		if (pSeq->JointsLoad() == S_FALSE)
			break;
	}
	input.close();
	Invalidate(-1);

	return S_OK;
}
int CSequenceHolder::GetSeqCount(){
	return m_nNumSeq;	
}